# Copyright (c) 2015 Intel Corpration
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools

from sahara.plugins import exceptions as ex
from sahara.plugins import testutils as tu
from sahara_plugin_cdh.tests.unit import base

ivse = ex.InvalidVolumeSizeException
icce = ex.InvalidComponentCountException
rsme = ex.RequiredServiceMissingException
icte = ex.InvalidClusterTopology
nnce = ex.NameNodeHAConfigurationError
rmce = ex.ResourceManagerHAConfigurationError


def make_ng_dict_with_inst(counter, name, flavor, processes, count,
                           instances=None, volumes_size=None,
                           node_configs=None, **kwargs):
    if not instances:
        instances = []
        for i in range(count):
            n = next(counter)
            instance = tu.make_inst_dict("id{0}".format(n),
                                         "fake_inst{0}".format(n),
                                         management_ip='1.2.3.{0}'.format(n))
            instances.append(instance)
    return tu.make_ng_dict(name, flavor, processes, count, instances,
                           volumes_size, node_configs, **kwargs)


def get_fake_cluster_with_process(processes=None,
                                  provided_ng_list=None, **kwargs):
    processes = processes or {}
    provided_ng_list = provided_ng_list or []
    inst_counter = itertools.count(start=0)
    ng_counter = itertools.count(start=0)
    ng_id_counter = itertools.count(start=0)

    # default
    mng_ng = ('manager_ng', 1, ['CLOUDERA_MANAGER'], 1)

    mst_ng = ('master_ng', 1, ['HDFS_NAMENODE',
                               'HDFS_SECONDARYNAMENODE',
                               'YARN_RESOURCEMANAGER',
                               'YARN_JOBHISTORY',
                               ], 1)

    wkrs_ng = ('worker_ng', 1, ['HDFS_DATANODE',
                                'YARN_NODEMANAGER'], 3)

    basic_ng_list = [mng_ng, mst_ng, wkrs_ng]

    # if in default_ng_list, change it
    if 'CLOUDERA_MANAGER' in processes:
        if processes['CLOUDERA_MANAGER'] == 0:
            basic_ng_list.remove(mng_ng)
        else:
            processes['CLOUDERA_MANAGER'] -= 1

    for process in mst_ng[2]:
        if process in processes:
            if processes[process] == 0:
                mst_ng[2].remove(process)
            else:
                processes[process] -= 1

    # only consider processes set to 0
    for process in wkrs_ng[2]:
        if process in processes:
            if processes[process] == 0:
                wkrs_ng[2].remove(process)

    other_ng_list = []
    for process, count in processes.items():
        if count:
            ng = ('service_ng{0}'.format(next(ng_counter)),
                  1, [process], count)
            other_ng_list.append(ng)

    ng_list = basic_ng_list + other_ng_list + provided_ng_list

    ng_dict_list = [make_ng_dict_with_inst(
        inst_counter, *args,
        id="ng_id{0}".format(next(ng_id_counter)))
        for args in ng_list]

    return tu.create_cluster('test_cluster', 1, 'cdh',
                             '5', ng_dict_list, **kwargs)


class BaseValidationTestCase(base.SaharaTestCase):

    def _test_cluster_validation(self, exception, processes,
                                 provided_ng_list=None, kwargs_dict=None):
        provided_ng_list = provided_ng_list or []
        kwargs_dict = kwargs_dict or {}
        cluster = get_fake_cluster_with_process(processes, provided_ng_list,
                                                **kwargs_dict)
        if exception:
            self.assertRaises(exception,
                              self.module.validate_cluster_creating, cluster)
        else:
            self.module.validate_cluster_creating(cluster)

    def setUp(self):
        super(BaseValidationTestCase, self).setUp()
        self.module = None

    def _get_test_cases(self):
        """return cases with [exception expected, progresses, configs]."""
        cases = [
            [icce, {'CLOUDERA_MANAGER': 0}],
            [None, {'CLOUDERA_MANAGER': 1}],
            [icce, {'CLOUDERA_MANAGER': 2}],
            [icce, {'HDFS_NAMENODE': 0}],
            [None, {'HDFS_NAMENODE': 1}],
            [icce, {'HDFS_NAMENODE': 2}],
            [icce, {'HDFS_SECONDARYNAMENODE': 0}],
            [None, {'HDFS_SECONDARYNAMENODE': 1}],
            [icce, {'HDFS_SECONDARYNAMENODE': 2}],
            [None, {}, [],
                {'cluster_configs': {'HDFS': {'dfs_replication': 2}}}],
            [None, {}, [],
                {'cluster_configs': {'HDFS': {'dfs_replication': 3}}}],
            [icce, {}, [],
                {'cluster_configs': {'HDFS': {'dfs_replication': 4}}}],
            [ivse, {}, [(
                'worker_ng_vol', 1, ['HDFS_DATANODE', 'YARN_NODEMANAGER'],
                3, None, 20,
                {'DATANODE': {'dfs_datanode_du_reserved': 22548578304}})]],
            [None, {}, [(
                'worker_ng_vol', 1, ['HDFS_DATANODE', 'YARN_NODEMANAGER'],
                3, None, 22,
                {'DATANODE': {'dfs_datanode_du_reserved': 22548578304}})]],
            [None, {'YARN_RESOURCEMANAGER': 1}],
            [icce, {'YARN_RESOURCEMANAGER': 2}],
            [None, {'YARN_JOBHISTORY': 1}],
            [icce, {'YARN_JOBHISTORY': 2}],
            [rsme, {'YARN_JOBHISTORY': 0, 'YARN_RESOURCEMANAGER': 1}],
            [rsme, {'YARN_RESOURCEMANAGER': 0, 'YARN_NODEMANAGER': 3}],
            [None, {'YARN_RESOURCEMANAGER': 0, 'YARN_NODEMANAGER': 0}],
            [None, {'OOZIE_SERVER': 1}],
            [icce, {'OOZIE_SERVER': 2}],
            [rsme, {'YARN_NODEMANAGER': 0, 'OOZIE_SERVER': 1}],
            [rsme, {'YARN_JOBHISTORY': 0, 'OOZIE_SERVER': 1}],
            [rsme, {'HIVE_SERVER2': 0, 'HIVE_METASTORE': 1}],
            [rsme, {'HIVE_METASTORE': 0, 'HIVE_SERVER2': 1}],
            [rsme, {'HIVE_METASTORE': 0, 'HIVE_SERVER2': 0,
                    'HIVE_WEBHCAT': 1}],
            [None, {'HUE_SERVER': 1, 'OOZIE_SERVER': 1, 'HIVE_METASTORE': 1,
                    'HIVE_SERVER2': 1}],
            [icce, {'HUE_SERVER': 2, 'OOZIE_SERVER': 1, 'HIVE_METASTORE': 1,
                    'HIVE_SERVER2': 1}],
            [rsme, {'HUE_SERVER': 1, 'OOZIE_SERVER': 0}],
            [rsme, {'HUE_SERVER': 1, 'OOZIE_SERVER': 1, 'HIVE_METASTORE': 0}],
            [None, {'SPARK_YARN_HISTORY_SERVER': 0}],
            [None, {'SPARK_YARN_HISTORY_SERVER': 1}],
            [icce, {'SPARK_YARN_HISTORY_SERVER': 2}],
            [rsme, {'HBASE_MASTER': 1, 'ZOOKEEPER_SERVER': 0}],
            [icce, {'HBASE_MASTER': 1, 'ZOOKEEPER_SERVER': 1,
                    'HBASE_REGIONSERVER': 0}],
            [None, {'HBASE_MASTER': 1, 'ZOOKEEPER_SERVER': 1,
                    'HBASE_REGIONSERVER': 1}],
            [icce, {'HBASE_MASTER': 0, 'HBASE_REGIONSERVER': 1}],
            [None, {}]
        ]

        disable_anti_affinity = {'cluster_configs': {'general': {
            'Require Anti Affinity': False}}}
        cases += [
            [None, {'HDFS_JOURNALNODE': 3,
                    'ZOOKEEPER_SERVER': 1}, [], disable_anti_affinity],
            [rsme, {'HDFS_JOURNALNODE': 3,
                    'ZOOKEEPER_SERVER': 0}, [], disable_anti_affinity],
            [icce, {'HDFS_JOURNALNODE': 4,
                    'ZOOKEEPER_SERVER': 1}, [], disable_anti_affinity],
            [icce, {'HDFS_JOURNALNODE': 2,
                    'ZOOKEEPER_SERVER': 1}, [], disable_anti_affinity],
            [nnce, {'HDFS_JOURNALNODE': 3, 'ZOOKEEPER_SERVER': 1}, [],
             {'anti_affinity': ['HDFS_SECONDARYNAMENODE']}],
            [nnce, {'HDFS_JOURNALNODE': 3, 'ZOOKEEPER_SERVER': 1}, [],
             {'anti_affinity': ['HDFS_NAMENODE']}],
            [None, {'HDFS_JOURNALNODE': 3, 'ZOOKEEPER_SERVER': 1}, [],
             {'anti_affinity': ['HDFS_NAMENODE', 'HDFS_SECONDARYNAMENODE']}],
            [None, {'YARN_STANDBYRM': 1,
                    'ZOOKEEPER_SERVER': 1}, [], disable_anti_affinity],
            [icce, {'YARN_STANDBYRM': 2,
                    'ZOOKEEPER_SERVER': 1}, [], disable_anti_affinity],
            [rsme, {'YARN_STANDBYRM': 1,
                    'ZOOKEEPER_SERVER': 0}, [], disable_anti_affinity],
            [rmce, {'YARN_STANDBYRM': 1, 'ZOOKEEPER_SERVER': 1}, [],
             {'anti_affinity': ['YARN_RESOURCEMANAGER']}],
            [rmce, {'YARN_STANDBYRM': 1, 'ZOOKEEPER_SERVER': 1}, [],
             {'anti_affinity': ['YARN_STANDBYRM']}],
            [None, {'YARN_STANDBYRM': 1, 'ZOOKEEPER_SERVER': 1}, [],
             {'anti_affinity': ['YARN_STANDBYRM', 'YARN_RESOURCEMANAGER']}],
        ]

        cases += [
            [None, {'FLUME_AGENT': 1}],
            [icce, {'ZOOKEEPER_SERVER': 1, 'SENTRY_SERVER': 2}],
            [None, {'ZOOKEEPER_SERVER': 1, 'SENTRY_SERVER': 1}],
            [rsme, {'ZOOKEEPER_SERVER': 0, 'SENTRY_SERVER': 1}],
            [None, {'ZOOKEEPER_SERVER': 1, 'SOLR_SERVER': 1}],
            [rsme, {'ZOOKEEPER_SERVER': 0, 'SOLR_SERVER': 1}],
            [None, {'YARN_NODEMANAGER': 1, 'YARN_JOBHISTORY': 1,
                    'SQOOP_SERVER': 1}],
            [rsme, {'YARN_NODEMANAGER': 0, 'YARN_JOBHISTORY': 1,
                    'SQOOP_SERVER': 1}],
            [rsme, {'YARN_NODEMANAGER': 1, 'YARN_JOBHISTORY': 0,
                    'SQOOP_SERVER': 1}],
            # HBASE_MASTER AND HBASE_REGIONSERVER depend circularly
            [None, {'ZOOKEEPER_SERVER': 1, 'SOLR_SERVER': 1,
                    'HBASE_MASTER': 1, 'HBASE_INDEXER': 1,
                    'HBASE_REGIONSERVER': 1}],
            [rsme, {'ZOOKEEPER_SERVER': 0, 'SOLR_SERVER': 1,
                    'HBASE_MASTER': 1, 'HBASE_INDEXER': 1,
                    'HBASE_REGIONSERVER': 1}],
            [rsme, {'ZOOKEEPER_SERVER': 1, 'SOLR_SERVER': 0,
                    'HBASE_MASTER': 1, 'HBASE_INDEXER': 1,
                    'HBASE_REGIONSERVER': 1}],
            [rsme, {'ZOOKEEPER_SERVER': 1, 'SOLR_SERVER': 1,
                    'HBASE_MASTER': 0, 'HBASE_INDEXER': 1}],
        ]

        worker_with_implama = ('worker_ng', 1, ['HDFS_DATANODE',
                                                'YARN_NODEMANAGER',
                                                'IMPALAD'], 3)
        cases += [
            [None, {'IMPALA_CATALOGSERVER': 1, 'IMPALA_STATESTORE': 1,
                    'HIVE_METASTORE': 1, 'HIVE_SERVER2': 1,
                    'HDFS_DATANODE': 0, 'YARN_NODEMANAGER': 0},
             [worker_with_implama]],
            [icte, {'IMPALA_CATALOGSERVER': 1, 'IMPALA_STATESTORE': 1,
                    'HIVE_METASTORE': 1, 'HIVE_SERVER2': 1},
             []],
            [icte, {'IMPALA_CATALOGSERVER': 1, 'IMPALA_STATESTORE': 1,
                    'HIVE_METASTORE': 1, 'HIVE_SERVER2': 1},
             [worker_with_implama]],
            [rsme, {'IMPALA_CATALOGSERVER': 1, 'IMPALA_STATESTORE': 0,
                    'HIVE_METASTORE': 1, 'HIVE_SERVER2': 1,
                    'HDFS_DATANODE': 0, 'YARN_NODEMANAGER': 0},
             [worker_with_implama]],
            [rsme, {'IMPALA_CATALOGSERVER': 1, 'IMPALA_STATESTORE': 1,
                    'HIVE_METASTORE': 0, 'HIVE_SERVER2': 1,
                    'HDFS_DATANODE': 0, 'YARN_NODEMANAGER': 0},
             [worker_with_implama]]

        ]

        cases += [
            [None, {'KMS': 1}],
            [icce, {'KMS': 2}]
        ]

        return cases

    def test_validate_cluster_creating(self):
        cases = self._get_test_cases()

        for case in cases:
            self._test_cluster_validation(*case)

    def test_validate_additional_ng_scaling(self):
        # valid scaling
        cluster = get_fake_cluster_with_process({})
        additional = {cluster.node_groups[2].id: 4}
        self.module.validate_additional_ng_scaling(cluster, additional)

        # node processes unscalable
        cluster = get_fake_cluster_with_process({})
        additional = {cluster.node_groups[0].id: 2}
        self.assertRaises(ex.NodeGroupCannotBeScaled,
                          self.module.validate_additional_ng_scaling, cluster,
                          additional)

        # scaling without resourcemanager
        cluster = get_fake_cluster_with_process({"YARN_RESOURCEMANAGER": 0})
        additional = {cluster.node_groups[2].id: 4}
        self.assertRaises(ex.NodeGroupCannotBeScaled,
                          self.module.validate_additional_ng_scaling, cluster,
                          additional)

    def test_validate_existing_ng_scaling(self):
        # valid scaling, add datanodes
        cluster = get_fake_cluster_with_process({})
        existing = {cluster.node_groups[2].id: 4}
        self.module.validate_existing_ng_scaling(cluster, existing)

        # node processes unscalable
        cluster = get_fake_cluster_with_process({})
        existing = {cluster.node_groups[0].id: 2}
        self.assertRaises(ex.NodeGroupCannotBeScaled,
                          self.module.validate_existing_ng_scaling, cluster,
                          existing)

        # datanode count less than replicas
        cfg = {'HDFS': {'dfs_replication': 3}}
        cluster = get_fake_cluster_with_process({},
                                                cluster_configs=cfg)
        existing = {cluster.node_groups[2].id: 2}
        self.assertRaises(ex.ClusterCannotBeScaled,
                          self.module.validate_existing_ng_scaling, cluster,
                          existing)

        # datanode count more than replicas
        cfg = {'HDFS': {'dfs_replication': 2}}
        cluster = get_fake_cluster_with_process({},
                                                cluster_configs=cfg)
        existing = {cluster.node_groups[2].id: 2}
        self.module.validate_existing_ng_scaling(cluster, existing)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import exceptions as ex
from sahara.plugins import utils as u
from sahara_plugin_cdh.i18n import _


class Validator(object):
    PU = None

    @classmethod
    def validate_cluster_creating(cls, cluster):
        cls._basic_validation(cluster)
        cls._oozie_validation(cluster)
        cls._hive_validation(cluster)
        cls._hue_validation(cluster)
        cls._hbase_validation(cluster)

        cls._flume_validation(cluster)
        cls._sentry_validation(cluster)
        cls._solr_validation(cluster)
        cls._sqoop_validation(cluster)
        cls._hbase_indexer_validation(cluster)
        cls._impala_validation(cluster)
        cls._kms_validation(cluster)

        cls._hdfs_ha_validation(cluster)
        cls._yarn_ha_validation(cluster)

    @classmethod
    def _basic_validation(cls, cluster):

        mng_count = cls.get_inst_count(cluster, 'CLOUDERA_MANAGER')
        if mng_count != 1:
            raise ex.InvalidComponentCountException('CLOUDERA_MANAGER',
                                                    1, mng_count)

        nn_count = cls.get_inst_count(cluster, 'HDFS_NAMENODE')
        if nn_count != 1:
            raise ex.InvalidComponentCountException(
                'HDFS_NAMENODE', 1, nn_count)

        snn_count = cls.get_inst_count(cluster, 'HDFS_SECONDARYNAMENODE')
        if snn_count != 1:
            raise ex.InvalidComponentCountException('HDFS_SECONDARYNAMENODE',
                                                    1, snn_count)
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')
        replicas = cls.PU.get_config_value('HDFS', 'dfs_replication', cluster)
        if dn_count < replicas:
            raise ex.InvalidComponentCountException(
                'HDFS_DATANODE', replicas, dn_count,
                _('Number of datanodes must be not'
                  ' less than dfs_replication.'))

        du_reserved = cls.PU.get_config_value(
            'DATANODE', 'dfs_datanode_du_reserved', cluster)
        du_reserved = du_reserved/1073741824.
        for node_group in cluster.node_groups:
            volume_size = node_group.volumes_size
            if volume_size and volume_size < du_reserved:
                raise ex.InvalidVolumeSizeException(volume_size, du_reserved)

        rm_count = cls.get_inst_count(cluster, 'YARN_RESOURCEMANAGER')
        if rm_count > 1:
            raise ex.InvalidComponentCountException('YARN_RESOURCEMANAGER',
                                                    _('0 or 1'), rm_count)

        hs_count = cls.get_inst_count(cluster, 'YARN_JOBHISTORY')
        if hs_count > 1:
            raise ex.InvalidComponentCountException('YARN_JOBHISTORY',
                                                    _('0 or 1'),
                                                    hs_count)

        if rm_count > 0 and hs_count < 1:
            raise ex.RequiredServiceMissingException(
                'YARN_JOBHISTORY', required_by='YARN_RESOURCEMANAGER')

        nm_count = cls.get_inst_count(cluster, 'YARN_NODEMANAGER')
        if rm_count == 0:
            if nm_count > 0:
                raise ex.RequiredServiceMissingException(
                    'YARN_RESOURCEMANAGER', required_by='YARN_NODEMANAGER')

    @classmethod
    def _oozie_validation(cls, cluster):

        oo_count = cls.get_inst_count(cluster, 'OOZIE_SERVER')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')
        nm_count = cls.get_inst_count(cluster, 'YARN_NODEMANAGER')
        hs_count = cls.get_inst_count(cluster, 'YARN_JOBHISTORY')

        if oo_count > 1:
            raise ex.InvalidComponentCountException(
                'OOZIE_SERVER', _('0 or 1'), oo_count)

        if oo_count == 1:
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='OOZIE_SERVER')

            if nm_count < 1:
                raise ex.RequiredServiceMissingException(
                    'YARN_NODEMANAGER', required_by='OOZIE_SERVER')

            if hs_count != 1:
                raise ex.RequiredServiceMissingException(
                    'YARN_JOBHISTORY', required_by='OOZIE_SERVER')

    @classmethod
    def _hive_validation(cls, cluster):
        hms_count = cls.get_inst_count(cluster, 'HIVE_METASTORE')
        hvs_count = cls.get_inst_count(cluster, 'HIVE_SERVER2')
        whc_count = cls.get_inst_count(cluster, 'HIVE_WEBHCAT')
        rm_count = cls.get_inst_count(cluster, 'YARN_RESOURCEMANAGER')

        if hms_count and rm_count < 1:
            raise ex.RequiredServiceMissingException(
                'YARN_RESOURCEMANAGER', required_by='HIVE_METASTORE')

        if hms_count and not hvs_count:
            raise ex.RequiredServiceMissingException(
                'HIVE_SERVER2', required_by='HIVE_METASTORE')

        if hvs_count and not hms_count:
            raise ex.RequiredServiceMissingException(
                'HIVE_METASTORE', required_by='HIVE_SERVER2')

        if whc_count and not hms_count:
            raise ex.RequiredServiceMissingException(
                'HIVE_METASTORE', required_by='WEBHCAT')

    @classmethod
    def _hue_validation(cls, cluster):
        hue_count = cls.get_inst_count(cluster, 'HUE_SERVER')
        if hue_count > 1:
            raise ex.InvalidComponentCountException(
                'HUE_SERVER', _('0 or 1'), hue_count)

        shs_count = cls.get_inst_count(cluster, 'SPARK_YARN_HISTORY_SERVER')
        hms_count = cls.get_inst_count(cluster, 'HIVE_METASTORE')
        oo_count = cls.get_inst_count(cluster, 'OOZIE_SERVER')
        rm_count = cls.get_inst_count(cluster, 'YARN_RESOURCEMANAGER')

        if shs_count > 1:
            raise ex.InvalidComponentCountException(
                'SPARK_YARN_HISTORY_SERVER',
                _('0 or 1'), shs_count)
        if shs_count and not rm_count:
            raise ex.RequiredServiceMissingException(
                'YARN_RESOURCEMANAGER',
                required_by='SPARK_YARN_HISTORY_SERVER')

        if oo_count < 1 and hue_count:
            raise ex.RequiredServiceMissingException(
                'OOZIE_SERVER', required_by='HUE_SERVER')

        if hms_count < 1 and hue_count:
            raise ex.RequiredServiceMissingException(
                'HIVE_METASTORE', required_by='HUE_SERVER')

    @classmethod
    def _hbase_validation(cls, cluster):
        hbm_count = cls.get_inst_count(cluster, 'HBASE_MASTER')
        hbr_count = cls.get_inst_count(cluster, 'HBASE_REGIONSERVER')
        zk_count = cls.get_inst_count(cluster, 'ZOOKEEPER_SERVER')

        if hbm_count == 1:
            if zk_count < 1:
                raise ex.RequiredServiceMissingException(
                    'ZOOKEEPER', required_by='HBASE')
            if hbr_count < 1:
                raise ex.InvalidComponentCountException(
                    'HBASE_REGIONSERVER', _('at least 1'), hbr_count)
        elif hbm_count > 1:
            raise ex.InvalidComponentCountException('HBASE_MASTER',
                                                    _('0 or 1'), hbm_count)
        elif hbr_count >= 1:
            raise ex.InvalidComponentCountException('HBASE_MASTER',
                                                    _('at least 1'), hbm_count)

    @classmethod
    def validate_additional_ng_scaling(cls, cluster, additional):
        rm = cls.PU.get_resourcemanager(cluster)
        scalable_processes = cls._get_scalable_processes()

        for ng_id in additional:
            ng = u.get_by_id(cluster.node_groups, ng_id)
            if not set(ng.node_processes).issubset(scalable_processes):
                msg = _("CDH plugin cannot scale nodegroup with processes: "
                        "%(processes)s")
                raise ex.NodeGroupCannotBeScaled(
                    ng.name, msg % {'processes': ' '.join(ng.node_processes)})

            if not rm and 'YARN_NODEMANAGER' in ng.node_processes:
                msg = _("CDH plugin cannot scale node group with processes "
                        "which have no master-processes run in cluster")
                raise ex.NodeGroupCannotBeScaled(ng.name, msg)

    @classmethod
    def validate_existing_ng_scaling(cls, cluster, existing):
        scalable_processes = cls._get_scalable_processes()
        dn_to_delete = 0
        for ng in cluster.node_groups:
            if ng.id in existing:
                if (ng.count > existing[ng.id] and
                        "HDFS_DATANODE" in ng.node_processes):
                    dn_to_delete += ng.count - existing[ng.id]

                if not set(ng.node_processes).issubset(scalable_processes):
                    msg = _("CDH plugin cannot scale nodegroup"
                            " with processes: %(processes)s")
                    raise ex.NodeGroupCannotBeScaled(
                        ng.name,
                        msg % {'processes': ' '.join(ng.node_processes)})

        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE') - dn_to_delete
        replicas = cls.PU.get_config_value('HDFS', 'dfs_replication', cluster)
        if dn_count < replicas:
            raise ex.ClusterCannotBeScaled(
                cluster, _('Number of datanodes must be not'
                           ' less than dfs_replication.'))

    @classmethod
    def _get_scalable_processes(cls):
        return ['HDFS_DATANODE', 'YARN_NODEMANAGER']

    @classmethod
    def _flume_validation(cls, cluster):
        a_count = cls.get_inst_count(cluster, 'FLUME_AGENT')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')

        if a_count >= 1:
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='FLUME_AGENT')

    @classmethod
    def _sentry_validation(cls, cluster):

        snt_count = cls.get_inst_count(cluster, 'SENTRY_SERVER')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')
        zk_count = cls.get_inst_count(cluster, 'ZOOKEEPER_SERVER')

        if snt_count > 1:
            raise ex.InvalidComponentCountException(
                'SENTRY_SERVER', _('0 or 1'), snt_count)
        if snt_count == 1:
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='SENTRY_SERVER')
            if zk_count < 1:
                raise ex.RequiredServiceMissingException(
                    'ZOOKEEPER', required_by='SENTRY_SERVER')

    @classmethod
    def _solr_validation(cls, cluster):
        slr_count = cls.get_inst_count(cluster, 'SOLR_SERVER')
        zk_count = cls.get_inst_count(cluster, 'ZOOKEEPER_SERVER')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')

        if slr_count >= 1:
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='SOLR_SERVER')
            if zk_count < 1:
                raise ex.RequiredServiceMissingException(
                    'ZOOKEEPER', required_by='SOLR_SERVER')

    @classmethod
    def _sqoop_validation(cls, cluster):

        s2s_count = cls.get_inst_count(cluster, 'SQOOP_SERVER')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')
        hs_count = cls.get_inst_count(cluster, 'YARN_JOBHISTORY')
        nm_count = cls.get_inst_count(cluster, 'YARN_NODEMANAGER')

        if s2s_count > 1:
            raise ex.InvalidComponentCountException(
                'SQOOP_SERVER', _('0 or 1'), s2s_count)
        if s2s_count == 1:
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='SQOOP_SERVER')
            if nm_count < 1:
                raise ex.RequiredServiceMissingException(
                    'YARN_NODEMANAGER', required_by='SQOOP_SERVER')
            if hs_count != 1:
                raise ex.RequiredServiceMissingException(
                    'YARN_JOBHISTORY', required_by='SQOOP_SERVER')

    @classmethod
    def _hbase_indexer_validation(cls, cluster):

        lhbi_count = cls.get_inst_count(cluster, 'HBASE_INDEXER')
        zk_count = cls.get_inst_count(cluster, 'ZOOKEEPER_SERVER')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')
        slr_count = cls.get_inst_count(cluster, 'SOLR_SERVER')
        hbm_count = cls.get_inst_count(cluster, 'HBASE_MASTER')

        if lhbi_count >= 1:
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='HBASE_INDEXER')
            if zk_count < 1:
                raise ex.RequiredServiceMissingException(
                    'ZOOKEEPER', required_by='HBASE_INDEXER')
            if slr_count < 1:
                raise ex.RequiredServiceMissingException(
                    'SOLR_SERVER', required_by='HBASE_INDEXER')
            if hbm_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HBASE_MASTER', required_by='HBASE_INDEXER')

    @classmethod
    def _impala_validation(cls, cluster):
        ics_count = cls.get_inst_count(cluster, 'IMPALA_CATALOGSERVER')
        iss_count = cls.get_inst_count(cluster, 'IMPALA_STATESTORE')
        id_count = cls.get_inst_count(cluster, 'IMPALAD')
        dn_count = cls.get_inst_count(cluster, 'HDFS_DATANODE')
        hms_count = cls.get_inst_count(cluster, 'HIVE_METASTORE')

        if ics_count > 1:
            raise ex.InvalidComponentCountException('IMPALA_CATALOGSERVER',
                                                    _('0 or 1'), ics_count)
        if iss_count > 1:
            raise ex.InvalidComponentCountException('IMPALA_STATESTORE',
                                                    _('0 or 1'), iss_count)
        if ics_count == 1:
            datanode_ng = u.get_node_groups(cluster, "HDFS_DATANODE")
            impalad_ng = u.get_node_groups(cluster, "IMPALAD")
            datanodes = set(ng.id for ng in datanode_ng)
            impalads = set(ng.id for ng in impalad_ng)

            if datanodes != impalads:
                raise ex.InvalidClusterTopology(
                    _("IMPALAD must be installed on every HDFS_DATANODE"))

            if iss_count != 1:
                raise ex.RequiredServiceMissingException(
                    'IMPALA_STATESTORE', required_by='IMPALA')
            if id_count < 1:
                raise ex.RequiredServiceMissingException(
                    'IMPALAD', required_by='IMPALA')
            if dn_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HDFS_DATANODE', required_by='IMPALA')
            if hms_count < 1:
                raise ex.RequiredServiceMissingException(
                    'HIVE_METASTORE', required_by='IMPALA')

    @classmethod
    def _kms_validation(cls, cluster):

        kms_count = cls.get_inst_count(cluster, 'KMS')
        if kms_count > 1:
            raise ex.InvalidComponentCountException('KMS',
                                                    _('0 or 1'), kms_count)

    @classmethod
    def _hdfs_ha_validation(cls, cluster):
        jn_count = cls.get_inst_count(cluster, 'HDFS_JOURNALNODE')
        zk_count = cls.get_inst_count(cluster, 'ZOOKEEPER_SERVER')

        require_anti_affinity = cls.PU.c_helper.get_required_anti_affinity(
            cluster)

        if jn_count > 0:
            if jn_count < 3:
                raise ex.InvalidComponentCountException('HDFS_JOURNALNODE',
                                                        _('not less than 3'),
                                                        jn_count)
            if not jn_count % 2:
                raise ex.InvalidComponentCountException('HDFS_JOURNALNODE',
                                                        _('be odd'), jn_count)
            if zk_count < 1:
                raise ex.RequiredServiceMissingException('ZOOKEEPER',
                                                         required_by='HDFS HA')
            if require_anti_affinity:
                if 'HDFS_SECONDARYNAMENODE' not in \
                        cls._get_anti_affinity(cluster):
                    raise ex.NameNodeHAConfigurationError(
                        _('HDFS_SECONDARYNAMENODE should be enabled '
                          'in anti_affinity.'))
                if 'HDFS_NAMENODE' not in cls._get_anti_affinity(cluster):
                    raise ex.NameNodeHAConfigurationError(
                        _('HDFS_NAMENODE should be enabled in anti_affinity.'))

    @classmethod
    def _yarn_ha_validation(cls, cluster):
        rm_count = cls.get_inst_count(cluster, 'YARN_RESOURCEMANAGER')
        zk_count = cls.get_inst_count(cluster, 'ZOOKEEPER_SERVER')
        stdb_rm_count = cls.get_inst_count(cluster, 'YARN_STANDBYRM')

        require_anti_affinity = cls.PU.c_helper.get_required_anti_affinity(
            cluster)

        if stdb_rm_count > 1:
            raise ex.InvalidComponentCountException(
                'YARN_STANDBYRM', _('0 or 1'), stdb_rm_count)
        if stdb_rm_count > 0:
            if rm_count < 1:
                raise ex.RequiredServiceMissingException(
                    'YARN_RESOURCEMANAGER', required_by='RM HA')
            if zk_count < 1:
                raise ex.RequiredServiceMissingException(
                    'ZOOKEEPER', required_by='RM HA')
            if require_anti_affinity:
                if 'YARN_RESOURCEMANAGER' not in \
                        cls._get_anti_affinity(cluster):
                    raise ex.ResourceManagerHAConfigurationError(
                        _('YARN_RESOURCEMANAGER should be enabled in '
                          'anti_affinity.'))
                if 'YARN_STANDBYRM' not in cls._get_anti_affinity(cluster):
                    raise ex.ResourceManagerHAConfigurationError(
                        _('YARN_STANDBYRM should be'
                          ' enabled in anti_affinity.'))

    @classmethod
    def _get_anti_affinity(cls, cluster):
        return cluster.anti_affinity

    @classmethod
    def get_inst_count(cls, cluster, process):
        return sum([ng.count for ng in u.get_node_groups(cluster, process)])



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import testtools

from sahara.plugins import health_check_base as base_health
from sahara_plugin_cdh.plugins.cdh import health
from sahara_plugin_cdh.tests.unit import base as unit_base


class TestHealthCheck(unit_base.SaharaTestCase):
    def test_check_health_availability(self):
        cluster = mock.Mock(plugin_name='cdh', hadoop_version='5.11.0')
        self.assertTrue(health.ClouderaManagerHealthCheck(
            cluster, mock.Mock()).is_available())

        cluster = mock.Mock(plugin_name='cdh', hadoop_version='5.11.0')
        self.assertTrue(health.ServiceHealthCheck(
            cluster, mock.Mock(), mock.Mock()).is_available())

    def _base_negative_testcase(self, cdh_response_mock, return_value, msg,
                                col, service, postfix=None):
        if not postfix:
            postfix = ""
        cdh_response_mock.return_value = return_value
        exc = (base_health.YellowHealthError
               if col == 'YELLOW' else base_health.RedHealthError)
        with testtools.ExpectedException(exc):
            try:
                pr = health.HealthStatusProvider(mock.Mock(), mock.Mock())
                health.ServiceHealthCheck(
                    mock.Mock(), provider=pr, service=service).check_health()
            except Exception as e:
                msg = "%s%s" % (msg, postfix)
                all_message = "Cluster health is %(col)s. Reason: %(msg)s" % {
                    'col': col, 'msg': msg}
                self.assertEqual(all_message, str(e))
                raise

    @mock.patch('sahara_plugin_cdh.plugins.cdh.health.HealthStatusProvider.'
                'get_cloudera_health')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.health.HealthStatusProvider.'
                'get_important_services')
    def test_health_calculation(self, important_stuff, cdh_response):
        important_stuff.return_value = ['yarn01', 'hdfs01', 'oozie01']
        cdh_response.return_value = {
            'yarn01': {
                'summary': 'GOOD',
            },
            'hdfs01': {
                'summary': 'GOOD',
            }
        }
        msg = ("Cloudera Manager has responded that service is in "
               "the %s state")
        pr = health.HealthStatusProvider(mock.Mock(), mock.Mock())
        self.assertEqual(
            msg % 'GOOD', health.ServiceHealthCheck(
                mock.Mock(), provider=pr, service='hdfs01').check_health())

        self._base_negative_testcase(cdh_response, {
            'yarn01': {'summary': 'GOOD'},
            'hdfs01': {'summary': 'BAD'}
        }, msg % 'BAD', 'RED', service='hdfs01')

        self._base_negative_testcase(cdh_response, {
            'yarn01': {'summary': 'CONCERNING'},
            'hdfs01': {'summary': 'BAD'}
        }, msg % 'CONCERNING', 'YELLOW', service='yarn01')

        # not important service, only yellow health
        self._base_negative_testcase(cdh_response, {
            'yarn01': {'summary': 'CONCERNING'},
            'hdfs01': {'summary': 'BAD'},
            'some_service01': {'summary': 'BAD'}
        }, msg % 'BAD', 'YELLOW', service='some_service01')

        self._base_negative_testcase(cdh_response, {
            'yarn01': {'summary': 'UNKNOWN_STATE'},
            'hdfs01': {'summary': 'BAD'},
            'some_service01': {'summary': 'BAD'}
        }, msg % 'UNKNOWN_STATE', 'RED', service='yarn01')

        # test additional info
        postfix = (". The following checks did not "
                   "pass: SUPER_HEALTH_CHECK - BAD state")
        self._base_negative_testcase(cdh_response, {
            'yarn01': {'summary': 'UNKNOWN_STATE'},
            'hdfs01': {'summary': 'BAD', 'checks': [
                {'name': 'SUPER_HEALTH_CHECK', 'summary': 'BAD'}]},
            'some_service01': {'summary': 'BAD'}
        }, msg % 'BAD', 'RED', service='hdfs01', postfix=postfix)



# Copyright (c) 2014 Intel Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# The contents of this file are mainly copied from cm_api sources,
# released by Cloudera. Codes not used by Sahara CDH plugin are removed.
# You can find the original codes at
#
#     https://github.com/cloudera/cm_api/tree/master/python/src/cm_api
#
# To satisfy the pep8 and python3 tests, we did some changes to the codes.
# We also change some importings to use Sahara inherited classes.

from sahara_plugin_cdh.plugins.cdh.client import types

ROLES_PATH = "/clusters/%s/services/%s/roles"
CM_ROLES_PATH = "/cm/service/roles"


def _get_roles_path(cluster_name, service_name):
    if cluster_name:
        return ROLES_PATH % (cluster_name, service_name)
    else:
        return CM_ROLES_PATH


def _get_role_path(cluster_name, service_name, role_name):
    path = _get_roles_path(cluster_name, service_name)
    return "%s/%s" % (path, role_name)


def create_role(resource_root,
                service_name,
                role_type,
                role_name,
                host_id,
                cluster_name="default"):
    """Create a role

    :param resource_root: The root Resource object.
    :param service_name: Service name
    :param role_type: Role type
    :param role_name: Role name
    :param cluster_name: Cluster name
    :return: An ApiRole object
    """
    apirole = ApiRole(resource_root, role_name, role_type,
                      types.ApiHostRef(resource_root, host_id))
    return types.call(resource_root.post,
                      _get_roles_path(cluster_name, service_name),
                      ApiRole, True, data=[apirole])[0]


def get_role(resource_root, service_name, name, cluster_name="default"):
    """Lookup a role by name

    :param resource_root: The root Resource object.
    :param service_name: Service name
    :param name: Role name
    :param cluster_name: Cluster name
    :return: An ApiRole object
    """
    return _get_role(resource_root, _get_role_path(cluster_name,
                     service_name, name))


def _get_role(resource_root, path):
    return types.call(resource_root.get, path, ApiRole)


def get_all_roles(resource_root, service_name, cluster_name="default",
                  view=None):
    """Get all roles

    :param resource_root: The root Resource object.
    :param service_name: Service name
    :param cluster_name: Cluster name
    :return: A list of ApiRole objects.
    """
    return types.call(resource_root.get,
                      _get_roles_path(cluster_name, service_name), ApiRole,
                      True, params=(dict(view=view) if view else None))


def get_roles_by_type(resource_root, service_name, role_type,
                      cluster_name="default", view=None):
    """Get all roles of a certain type in a service

    :param resource_root: The root Resource object.
    :param service_name: Service name
    :param role_type: Role type
    :param cluster_name: Cluster name
    :return: A list of ApiRole objects.
    """
    roles = get_all_roles(resource_root, service_name, cluster_name, view)
    return [r for r in roles if r.type == role_type]


def delete_role(resource_root, service_name, name, cluster_name="default"):
    """Delete a role by name

    :param resource_root: The root Resource object.
    :param service_name: Service name
    :param name: Role name
    :param cluster_name: Cluster name
    :return: The deleted ApiRole object
    """
    return types.call(resource_root.delete,
                      _get_role_path(cluster_name, service_name, name),
                      ApiRole)


class ApiRole(types.BaseApiResource):
    _ATTRIBUTES = {
        'name': None,
        'type': None,
        'hostRef': types.Attr(types.ApiHostRef),
        'roleState': types.ROAttr(),
        'healthSummary': types.ROAttr(),
        'healthChecks': types.ROAttr(),
        'serviceRef': types.ROAttr(types.ApiServiceRef),
        'configStale': types.ROAttr(),
        'configStalenessStatus': types.ROAttr(),
        'haStatus': types.ROAttr(),
        'roleUrl': types.ROAttr(),
        'commissionState': types.ROAttr(),
        'maintenanceMode': types.ROAttr(),
        'maintenanceOwners': types.ROAttr(),
        'roleConfigGroupRef': types.ROAttr(types.ApiRoleConfigGroupRef),
        'zooKeeperServerMode': types.ROAttr(),
    }

    def __init__(self, resource_root, name=None, type=None, hostRef=None):
        types.BaseApiObject.init(self, resource_root, locals())

    def __str__(self):
        return ("<ApiRole>: %s (cluster: %s; service: %s)"
                % (self.name, self.serviceRef.clusterName,
                   self.serviceRef.serviceName))

    def _path(self):
        return _get_role_path(self.serviceRef.clusterName,
                              self.serviceRef.serviceName,
                              self.name)

    def _get_log(self, log):
        path = "%s/logs/%s" % (self._path(), log)
        return self._get_resource_root().get(path)

    def get_commands(self, view=None):
        """Retrieve a list of running commands for this role

        :param view: View to materialize ('full' or 'summary')
        :return: A list of running commands.
        """
        return self._get("commands", types.ApiCommand, True,
                         params=(dict(view=view) if view else None))

    def get_config(self, view=None):
        """Retrieve the role's configuration

        The 'summary' view contains strings as the dictionary values. The full
        view contains types.ApiConfig instances as the values.

        :param view: View to materialize ('full' or 'summary')
        :return: Dictionary with configuration data.
        """
        return self._get_config("config", view)

    def update_config(self, config):
        """Update the role's configuration

        :param config: Dictionary with configuration to update.
        :return: Dictionary with updated configuration.
        """
        return self._update_config("config", config)



# Copyright (c) 2014 Intel Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# The contents of this file are mainly copied from cm_api sources,
# released by Cloudera. Codes not used by Sahara CDH plugin are removed.
# You can find the original codes at
#
#     https://github.com/cloudera/cm_api/tree/master/python/src/cm_api
#
# To satisfy the pep8 and python3 tests, we did some changes to the codes.
# We also change some importings to use Sahara inherited classes.

import posixpath
import socket

from oslo_log import log as logging
from oslo_serialization import jsonutils as json
import urllib

from sahara.plugins import context
from sahara_plugin_cdh.i18n import _
from sahara_plugin_cdh.plugins.cdh import exceptions as ex

LOG = logging.getLogger(__name__)


class Resource(object):
    """Base Resource

    Encapsulates a resource, and provides actions to invoke on it.
    """
    def __init__(self, client, relpath=""):
        """Constructor method

        :param client: A Client object.
        :param relpath: The relative path of the resource.
        """
        self._client = client
        self._path = relpath.strip('/')
        self.retries = 3
        self.retry_sleep = 3

    @property
    def base_url(self):
        return self._client.base_url

    def _join_uri(self, relpath):
        if relpath is None:
            return self._path
        return self._path + posixpath.normpath('/' + relpath)

    def invoke(self, method, relpath=None, params=None, data=None,
               headers=None):
        """Invoke an API method

        :return: Raw body or JSON dictionary (if response content type is
                 JSON).
        """
        path = self._join_uri(relpath)
        resp = self._client.execute(method,
                                    path,
                                    params=params,
                                    data=data,
                                    headers=headers)
        try:
            body = resp.read()
        except Exception as ex:
            raise ex.CMApiException(
                _("Command %(method)s %(path)s failed: %(msg)s")
                % {'method': method, 'path': path, 'msg': str(ex)})

        LOG.debug("{method} got response: {body}".format(method=method,
                                                         body=body[:32]))
        # Is the response application/json?
        if (len(body) != 0 and
            self._get_content_maintype(resp.info()) == "application"
                and self._get_content_subtype(resp.info()) == "json"):
            try:
                json_dict = json.loads(body)
                return json_dict
            except Exception:
                LOG.error('JSON decode error: {body}'.format(body=body))
                raise
        else:
            return body

    def get(self, relpath=None, params=None):
        """Invoke the GET method on a resource

        :param relpath: Optional. A relative path to this resource's path.
        :param params: Key-value data.

        :return: A dictionary of the JSON result.
        """
        for retry in range(self.retries + 1):
            if retry:
                context.sleep(self.retry_sleep)
            try:
                return self.invoke("GET", relpath, params)
            except (socket.error, urllib.error.URLError) as e:
                if "timed out" in str(e).lower():
                    if retry < self.retries:
                        LOG.warning("Timeout issuing GET request for "
                                    "{path}. Will retry".format(
                                        path=self._join_uri(relpath)))
                    else:
                        LOG.warning("Timeout issuing GET request for "
                                    "{path}. No retries left".format(
                                        path=self._join_uri(relpath)))
                else:
                    raise
        else:
            raise ex.CMApiException(_("Get retry max time reached."))

    def delete(self, relpath=None, params=None):
        """Invoke the DELETE method on a resource

        :param relpath: Optional. A relative path to this resource's path.
        :param params: Key-value data.

        :return: A dictionary of the JSON result.
        """
        return self.invoke("DELETE", relpath, params)

    def post(self, relpath=None, params=None, data=None, contenttype=None):
        """Invoke the POST method on a resource

        :param relpath: Optional. A relative path to this resource's path.
        :param params: Key-value data.
        :param data: Optional. Body of the request.
        :param contenttype: Optional.

        :return: A dictionary of the JSON result.
        """

        return self.invoke("POST", relpath, params, data,
                           self._make_headers(contenttype))

    def put(self, relpath=None, params=None, data=None, contenttype=None):
        """Invoke the PUT method on a resource

        :param relpath: Optional. A relative path to this resource's path.
        :param params: Key-value data.
        :param data: Optional. Body of the request.
        :param contenttype: Optional.

        :return: A dictionary of the JSON result.
        """
        return self.invoke("PUT", relpath, params, data,
                           self._make_headers(contenttype))

    def _make_headers(self, contenttype=None):
        if contenttype:
            return {'Content-Type': contenttype}
        return None

    def _get_content_maintype(self, info):
        try:
            return info.getmaintype()
        except AttributeError:
            return info.get_content_maintype()

    def _get_content_subtype(self, info):
        try:
            return info.getsubtype()
        except AttributeError:
            return info.get_content_subtype()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import kerberos


PACKAGES = [
    'cloudera-manager-agent',
    'cloudera-manager-daemons',
    'cloudera-manager-server',
    'cloudera-manager-server-db-2',
    'flume-ng',
    'hadoop-hdfs-datanode',
    'hadoop-hdfs-namenode',
    'hadoop-hdfs-secondarynamenode',
    'hadoop-kms'
    'hadoop-mapreduce',
    'hadoop-mapreduce-historyserver',
    'hadoop-yarn-nodemanager',
    'hadoop-yarn-resourcemanager',
    'hbase',
    'hbase-solr',
    'hive-hcatalog',
    'hive-metastore',
    'hive-server2',
    'hive-webhcat-server',
    'hue',
    'impala',
    'impala-server',
    'impala-state-store',
    'impala-catalog',
    'impala-shell',
    'kafka',
    'kafka-server'
    'keytrustee-keyprovider',
    'oozie',
    'oracle-j2sdk1.7',
    'sentry',
    'solr-server',
    'solr-doc',
    'search',
    'spark-history-server',
    'sqoop2',
    'unzip',
    'zookeeper'
]


def setup_kerberos_for_cluster(cluster, cloudera_utils):
    if kerberos.is_kerberos_security_enabled(cluster):
        manager = cloudera_utils.pu.get_manager(cluster)
        kerberos.deploy_infrastructure(cluster, manager)
        cloudera_utils.full_cluster_stop(cluster)
        kerberos.prepare_policy_files(cluster)
        cloudera_utils.push_kerberos_configs(cluster)
        cloudera_utils.full_cluster_start(cluster)
        kerberos.create_keytabs_for_map(
            cluster,
            {'hdfs': cloudera_utils.pu.get_hdfs_nodes(cluster),
             'spark': [cloudera_utils.pu.get_spark_historyserver(cluster)]})


def prepare_scaling_kerberized_cluster(cluster, cloudera_utils, instances):
    if kerberos.is_kerberos_security_enabled(cluster):
        server = None
        if not kerberos.using_existing_kdc(cluster):
            server = cloudera_utils.pu.get_manager(cluster)
        kerberos.setup_clients(cluster, server)
        kerberos.prepare_policy_files(cluster)
        # manager can correctly handle updating configs
        cloudera_utils.push_kerberos_configs(cluster)
        kerberos.create_keytabs_for_map(
            cluster,
            {'hdfs': cloudera_utils.pu.get_hdfs_nodes(cluster, instances)})


def get_open_ports(node_group):
    ports = [9000]  # for CM agent

    ports_map = {
        'CLOUDERA_MANAGER': [7180, 7182, 7183, 7432, 7184, 8084, 8086, 10101,
                             9997, 9996, 8087, 9998, 9999, 8085, 9995, 9994],
        'HDFS_NAMENODE': [8020, 8022, 50070, 50470],
        'HDFS_SECONDARYNAMENODE': [50090, 50495],
        'HDFS_DATANODE': [50010, 1004, 50075, 1006, 50020],
        'YARN_RESOURCEMANAGER': [8030, 8031, 8032, 8033, 8088],
        'YARN_STANDBYRM': [8030, 8031, 8032, 8033, 8088],
        'YARN_NODEMANAGER': [8040, 8041, 8042],
        'YARN_JOBHISTORY': [10020, 19888],
        'HIVE_METASTORE': [9083],
        'HIVE_SERVER2': [10000],
        'HUE_SERVER': [8888],
        'OOZIE_SERVER': [11000, 11001],
        'SPARK_YARN_HISTORY_SERVER': [18088],
        'ZOOKEEPER_SERVER': [2181, 3181, 4181, 9010],
        'HBASE_MASTER': [60000],
        'HBASE_REGIONSERVER': [60020],
        'FLUME_AGENT': [41414],
        'SENTRY_SERVER': [8038],
        'SOLR_SERVER': [8983, 8984],
        'SQOOP_SERVER': [8005, 12000],
        'KEY_VALUE_STORE_INDEXER': [],
        'IMPALA_CATALOGSERVER': [25020, 26000],
        'IMPALA_STATESTORE': [25010, 24000],
        'IMPALAD': [21050, 21000, 23000, 25000, 28000, 22000],
        'KMS': [16000, 16001],
        'JOURNALNODE': [8480, 8481, 8485]
    }

    for process in node_group.node_processes:
        if process in ports_map:
            ports.extend(ports_map[process])

    return ports



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools
from unittest import mock

from sahara_plugin_cdh.plugins.cdh import cloudera_utils as cu
from sahara_plugin_cdh.tests.unit import base
from sahara_plugin_cdh.tests.unit.plugins.cdh import utils as ctu


CU = cu.ClouderaUtils()


class ClouderaUtilsTestCase(base.SaharaTestCase):
    @mock.patch('sahara_plugin_cdh.plugins.cdh.cloudera_utils.ClouderaUtils.'
                'get_cloudera_cluster')
    def test_get_service(self, mock_get_cl_cluster):
        self.assertRaises(ValueError, CU.get_service_by_role, 'NAMENODE')

        cluster = ctu.get_fake_cluster()
        inst = cluster.node_groups[0].instances[0]
        mock_get_cl_cluster.return_value = None

        self.assertRaises(ValueError, CU.get_service_by_role, 'spam',
                          cluster)
        self.assertRaises(ValueError, CU.get_service_by_role, 'spam',
                          instance=inst)

        mock_get_cl_cluster.reset_mock()

        mock_get_service = mock.MagicMock()
        mock_get_service.get_service.return_value = mock.Mock()
        mock_get_cl_cluster.return_value = mock_get_service

        CU.get_service_by_role('NAMENODE', cluster)
        args = ((CU.HDFS_SERVICE_NAME,),)
        self.assertEqual(args, mock_get_service.get_service.call_args)

        mock_get_service.reset_mock()
        CU.get_service_by_role('JOBHISTORY', instance=inst)
        args = ((CU.YARN_SERVICE_NAME,),)
        self.assertEqual(args, mock_get_service.get_service.call_args)

        mock_get_service.reset_mock()
        CU.get_service_by_role('OOZIE_SERVER', cluster)
        args = ((CU.OOZIE_SERVICE_NAME,),)
        self.assertEqual(args, mock_get_service.get_service.call_args)

    def test_get_role_name(self):
        inst_mock = mock.Mock()
        inst_mock.hostname.return_value = 'spam-host'

        self.assertEqual('eggs_spam_host',
                         CU.pu.get_role_name(inst_mock, 'eggs'))

    @mock.patch('sahara_plugin_cdh.plugins.cdh.cloudera_utils.ClouderaUtils.'
                'get_cloudera_cluster')
    def test_get_service_by_role(self, get_cloudera_cluster):
        class Ob(object):
            def get_service(self, x):
                return x
        get_cloudera_cluster.return_value = Ob()
        roles = ['NAMENODE', 'DATANODE', 'SECONDARYNAMENODE', 'HDFS_GATEWAY',
                 'RESOURCEMANAGER', 'NODEMANAGER', 'JOBHISTORY',
                 'YARN_GATEWAY', 'OOZIE_SERVER', 'HIVESERVER2',
                 'HIVEMETASTORE', 'WEBHCAT', 'HUE_SERVER',
                 'SPARK_YARN_HISTORY_SERVER', 'SERVER', 'MASTER',
                 'REGIONSERVER']
        resps = ['hdfs01', 'hdfs01', 'hdfs01', 'hdfs01', 'yarn01', 'yarn01',
                 'yarn01', 'yarn01', 'oozie01', 'hive01', 'hive01', 'hive01',
                 'hue01', 'spark_on_yarn01', 'zookeeper01', 'hbase01',
                 'hbase01']
        provider = cu.ClouderaUtils()
        cluster = mock.Mock()
        for (role, resp) in zip(roles, resps):
            self.assertEqual(
                resp, provider.get_service_by_role(role, cluster=cluster))

        with testtools.ExpectedException(ValueError):
            provider.get_service_by_role('cat', cluster=cluster)



# Copyright (c) 2016 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from sahara.plugins import utils
from sahara_plugin_cdh.plugins.cdh.v5_7_0 import config_helper
from sahara_plugin_cdh.tests.unit import base
from sahara_plugin_cdh.tests.unit.plugins.cdh import utils as ctu

c_h = config_helper.ConfigHelperV570()

path_to_config = 'plugins/cdh/v5_7_0/resources/'
json_files = [
    'hdfs-service.json',
    'hdfs-namenode.json',
    'hdfs-datanode.json',
    'hdfs-secondarynamenode.json',
    'hdfs-gateway.json',
    'hdfs-journalnode.json',
    'yarn-service.json',
    'yarn-resourcemanager.json',
    'yarn-nodemanager.json',
    'yarn-jobhistory.json',
    'yarn-gateway.json',
    'oozie-service.json',
    'oozie-oozie_server.json',
    'hive-service.json',
    'hive-hivemetastore.json',
    'hive-hiveserver2.json',
    'hive-webhcat.json',
    'hue-service.json',
    'hue-hue_server.json',
    'spark-service.json',
    'spark-spark_yarn_history_server.json',
    'zookeeper-service.json',
    'zookeeper-server.json',
    'hbase-service.json',
    'hbase-master.json',
    'hbase-regionserver.json',
    'flume-service.json',
    'flume-agent.json',
    'sentry-service.json',
    'sentry-sentry_server.json',
    'solr-service.json',
    'solr-solr_server.json',
    'sqoop-service.json',
    'sqoop-sqoop_server.json',
    'ks_indexer-service.json',
    'ks_indexer-hbase_indexer.json',
    'impala-service.json',
    'impala-catalogserver.json',
    'impala-impalad.json',
    'impala-statestore.json',
    'kms-service.json',
    'kms-kms.json',
    'kafka-kafka_broker.json',
    'kafka-kafka_mirror_maker.json',
    'kafka-service.json'
]


class ConfigHelperTestCase(base.SaharaTestCase):

    def test_get_ng_plugin_configs(self):
        actual_configs = c_h._get_ng_plugin_configs()

        expected_configs = []
        for json_file in json_files:
            expected_configs += json.loads(
                utils.get_file_text(path_to_config + json_file,
                                    'sahara_plugin_cdh'))

        # compare names
        expected_names = set(i['name'] for i in expected_configs)
        actual_names = set(i.to_dict()['name'] for i in actual_configs)
        self.assertEqual(expected_names, actual_names)

    def test_get_cdh5_repo_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.CDH5_REPO_URL.default_value,
                         c_h.get_cdh5_repo_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.CDH5_REPO_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_cdh5_repo_url(cluster))

    def test_get_cdh5_key_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.CDH5_REPO_KEY_URL.default_value,
                         c_h.get_cdh5_key_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.CDH5_REPO_KEY_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_cdh5_key_url(cluster))

    def test_get_cm5_repo_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.CM5_REPO_URL.default_value,
                         c_h.get_cm5_repo_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.CM5_REPO_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_cm5_repo_url(cluster))

    def test_get_cm5_key_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.CM5_REPO_KEY_URL.default_value,
                         c_h.get_cm5_key_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.CM5_REPO_KEY_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_cm5_key_url(cluster))

    def test_is_swift_enabled(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertTrue(c_h.is_swift_enabled(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.ENABLE_SWIFT.name: False}})
        self.assertFalse(c_h.is_swift_enabled(cluster))

    def test_get_swift_lib_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.DEFAULT_SWIFT_LIB_URL,
                         c_h.get_swift_lib_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.SWIFT_LIB_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_swift_lib_url(cluster))

    def test_is_hbase_common_lib_enabled(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertTrue(c_h.is_hbase_common_lib_enabled(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general':
                             {c_h.ENABLE_HBASE_COMMON_LIB.name: False}})
        self.assertFalse(c_h.is_hbase_common_lib_enabled(cluster))

    def test_get_extjs_lib_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.DEFAULT_EXTJS_LIB_URL,
                         c_h.get_extjs_lib_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.EXTJS_LIB_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_extjs_lib_url(cluster))

    def test_get_kms_key_url(self):
        cluster = ctu.get_fake_cluster(cluster_configs={})
        self.assertEqual(c_h.KMS_REPO_KEY_URL.default_value,
                         c_h.get_kms_key_url(cluster))

        cluster = ctu.get_fake_cluster(
            cluster_configs={'general': {c_h.KMS_REPO_KEY_URL.name: 'spam'}})
        self.assertEqual('spam', c_h.get_kms_key_url(cluster))



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import edp
from sahara.plugins import utils as plugin_utils
from sahara_plugin_cdh.i18n import _
from sahara_plugin_cdh.plugins.cdh import commands as cmd
from sahara_plugin_cdh.plugins.cdh import deploy as common_deploy
from sahara_plugin_cdh.plugins.cdh.v5_11_0 import cloudera_utils as cu

CU = cu.ClouderaUtilsV5110()

PACKAGES = common_deploy.PACKAGES


def configure_cluster(cluster):
    instances = plugin_utils.get_instances(cluster)

    if not cmd.is_pre_installed_cdh(CU.pu.get_manager(cluster).remote()):
        CU.pu.configure_os(instances)
        CU.pu.install_packages(instances, PACKAGES)

    CU.pu.start_cloudera_agents(instances)
    CU.pu.start_cloudera_manager(cluster)
    CU.update_cloudera_password(cluster)
    CU.configure_rack_awareness(cluster)
    CU.await_agents(cluster, instances)
    CU.create_mgmt_service(cluster)
    CU.create_services(cluster)
    CU.configure_services(cluster)
    CU.configure_instances(instances, cluster)
    CU.deploy_configs(cluster)


@plugin_utils.event_wrapper(
    True, step=_("Start roles: NODEMANAGER, DATANODE"), param=('cluster', 0))
def _start_roles(cluster, instances):
    for instance in instances:
        if 'HDFS_DATANODE' in instance.node_group.node_processes:
            hdfs = CU.get_service_by_role('DATANODE', instance=instance)
            CU.start_roles(hdfs, CU.pu.get_role_name(instance, 'DATANODE'))

        if 'YARN_NODEMANAGER' in instance.node_group.node_processes:
            yarn = CU.get_service_by_role('NODEMANAGER', instance=instance)
            CU.start_roles(yarn, CU.pu.get_role_name(instance, 'NODEMANAGER'))


def scale_cluster(cluster, instances):
    if not instances:
        return

    if not cmd.is_pre_installed_cdh(instances[0].remote()):
        CU.pu.configure_os(instances)
        CU.pu.install_packages(instances, PACKAGES)

    CU.pu.start_cloudera_agents(instances)
    CU.await_agents(cluster, instances)
    CU.configure_rack_awareness(cluster)
    CU.configure_instances(instances, cluster)
    CU.update_configs(instances)
    common_deploy.prepare_scaling_kerberized_cluster(
        cluster, CU, instances)

    CU.pu.configure_swift(cluster, instances)
    _start_roles(cluster, instances)
    CU.refresh_datanodes(cluster)
    CU.refresh_yarn_nodes(cluster)
    CU.restart_stale_services(cluster)


def decommission_cluster(cluster, instances):
    dns = []
    dns_to_delete = []
    nms = []
    nms_to_delete = []
    for i in instances:
        if 'HDFS_DATANODE' in i.node_group.node_processes:
            dns.append(CU.pu.get_role_name(i, 'DATANODE'))
            dns_to_delete.append(
                CU.pu.get_role_name(i, 'HDFS_GATEWAY'))

        if 'YARN_NODEMANAGER' in i.node_group.node_processes:
            nms.append(CU.pu.get_role_name(i, 'NODEMANAGER'))
            nms_to_delete.append(
                CU.pu.get_role_name(i, 'YARN_GATEWAY'))

    if dns:
        CU.decommission_nodes(
            cluster, 'DATANODE', dns, dns_to_delete)

    if nms:
        CU.decommission_nodes(
            cluster, 'NODEMANAGER', nms, nms_to_delete)

    CU.delete_instances(cluster, instances)

    CU.refresh_datanodes(cluster)
    CU.refresh_yarn_nodes(cluster)
    CU.restart_stale_services(cluster)


@plugin_utils.event_wrapper(
    True, step=_("Prepare cluster"), param=('cluster', 0))
def _prepare_cluster(cluster):
    if CU.pu.get_oozie(cluster):
        CU.pu.install_extjs(cluster)

    if CU.pu.get_hive_metastore(cluster):
        CU.pu.configure_hive(cluster)

    if CU.pu.get_sentry(cluster):
        CU.pu.configure_sentry(cluster)


@plugin_utils.event_wrapper(
    True, step=_("Finish cluster starting"), param=('cluster', 0))
def _finish_cluster_starting(cluster):
    if CU.pu.get_hive_metastore(cluster):
        CU.pu.put_hive_hdfs_xml(cluster)

    server = CU.pu.get_hbase_master(cluster)
    if CU.pu.c_helper.is_hbase_common_lib_enabled(cluster) and server:
        with server.remote() as r:
            edp.create_hbase_common_lib(r)

    if CU.pu.get_flumes(cluster):
        flume = CU.get_service_by_role('AGENT', cluster)
        CU.start_service(flume)


def start_cluster(cluster):
    _prepare_cluster(cluster)

    CU.first_run(cluster)

    CU.pu.configure_swift(cluster)

    if len(CU.pu.get_jns(cluster)) > 0:
        CU.enable_namenode_ha(cluster)
        # updating configs for NameNode role on needed nodes
        CU.update_role_config(CU.pu.get_secondarynamenode(cluster),
                              'HDFS_NAMENODE')

    if CU.pu.get_stdb_rm(cluster):
        CU.enable_resourcemanager_ha(cluster)
        # updating configs for ResourceManager on needed nodes
        CU.update_role_config(CU.pu.get_stdb_rm(cluster), 'YARN_STANDBYRM')

    _finish_cluster_starting(cluster)

    common_deploy.setup_kerberos_for_cluster(cluster, CU)


def get_open_ports(node_group):
    ports = common_deploy.get_open_ports(node_group)
    return ports



# Copyright (c) 2015 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import base as pb
from sahara.plugins import edp
from sahara.plugins import exceptions as pl_ex
from sahara_plugin_cdh.plugins.cdh.v5_9_0 import edp_engine
from sahara_plugin_cdh.tests.unit import base as sahara_base
from sahara_plugin_cdh.tests.unit.plugins.cdh import utils as c_u


def get_cluster(version='5.9.0'):
    cluster = c_u.get_fake_cluster(plugin_name='CDH', hadoop_version=version)
    return cluster


class EdpEngineTestV590(sahara_base.SaharaTestCase):

    def setUp(self):
        super(EdpEngineTestV590, self).setUp()
        self.override_config('plugins', ['cdh'])
        pb.setup_plugins()

    def test_get_hdfs_user(self):
        eng = edp_engine.EdpOozieEngine(get_cluster())
        self.assertEqual('hdfs', eng.get_hdfs_user())

    @mock.patch('sahara.plugins.edp.create_dir_hadoop2')
    def test_create_hdfs_dir(self, create_dir_hadoop2):
        eng = edp_engine.EdpOozieEngine(get_cluster())
        remote = mock.Mock()
        dir_name = mock.Mock()
        eng.create_hdfs_dir(remote, dir_name)
        create_dir_hadoop2.assert_called_once_with(remote,
                                                   dir_name,
                                                   eng.get_hdfs_user())

    def test_get_oozie_server_uri(self):
        cluster = get_cluster()
        eng = edp_engine.EdpOozieEngine(cluster)
        uri = eng.get_oozie_server_uri(cluster)
        self.assertEqual("http://1.2.3.5:11000/oozie", uri)

    def test_get_name_node_uri(self):
        cluster = get_cluster()
        eng = edp_engine.EdpOozieEngine(cluster)
        uri = eng.get_name_node_uri(cluster)
        self.assertEqual("hdfs://master_inst.novalocal:8020", uri)

        # has HDFS_JOURNALNODE
        cluster = get_cluster()
        jns_node_group = mock.MagicMock()
        jns_node_group.node_processes = ['HDFS_JOURNALNODE']
        jns_node_group.instances = [mock.Mock()]
        list.append(cluster.node_groups, jns_node_group)
        uri = eng.get_name_node_uri(cluster)
        self.assertEqual("hdfs://nameservice01", uri)

    def test_get_resource_manager_uri(self):
        cluster = get_cluster()
        eng = edp_engine.EdpOozieEngine(cluster)
        uri = eng.get_resource_manager_uri(cluster)
        self.assertEqual("master_inst.novalocal:8032", uri)

    def test_get_oozie_server(self):
        cluster = get_cluster()
        eng = edp_engine.EdpOozieEngine(cluster)
        actual = eng.get_oozie_server(cluster)
        expect = cluster.node_groups[1].instances[0]
        self.assertEqual(expect, actual)

    @mock.patch('sahara.service.edp.oozie.engine.'
                'OozieJobEngine.validate_job_execution')
    def test_validate_job_execution(self, c):
        cluster = get_cluster()
        eng = edp_engine.EdpOozieEngine(cluster)
        eng.validate_job_execution(cluster, mock.Mock(), mock.Mock())

        # more than one oozie server
        dict.__setitem__(cluster.node_groups[1], 'count', 2)
        self.assertRaises(pl_ex.InvalidComponentCountException,
                          eng.validate_job_execution, cluster,
                          mock.Mock(), mock.Mock())

    @mock.patch(
        'sahara_plugin_cdh.plugins.cdh.confighints_helper.'
        'get_possible_hive_config_from',
        return_value={})
    def test_get_possible_job_config_hive(self,
                                          get_possible_hive_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_HIVE)
        get_possible_hive_config_from.assert_called_once_with(
            'plugins/cdh/v5_9_0/resources/hive-site.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara_plugin_cdh.plugins.cdh.v5_9_0.edp_engine.'
                'EdpOozieEngine')
    def test_get_possible_job_config_java(self, BaseCDHEdpOozieEngine):
        expected_config = {'job_config': {}}
        BaseCDHEdpOozieEngine.get_possible_job_config.return_value = (
            expected_config)
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_JAVA)
        BaseCDHEdpOozieEngine.get_possible_job_config.assert_called_once_with(
            edp.JOB_TYPE_JAVA)
        self.assertEqual(expected_config, actual_config)

    @mock.patch(
        'sahara_plugin_cdh.plugins.cdh.confighints_helper.'
        'get_possible_mapreduce_config_from',
        return_value={})
    def test_get_possible_job_config_mapreduce(
            self, get_possible_mapreduce_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_MAPREDUCE)
        get_possible_mapreduce_config_from.assert_called_once_with(
            'plugins/cdh/v5_9_0/resources/mapred-site.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch(
        'sahara_plugin_cdh.plugins.cdh.confighints_helper.'
        'get_possible_mapreduce_config_from',
        return_value={})
    def test_get_possible_job_config_mapreduce_streaming(
            self, get_possible_mapreduce_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_MAPREDUCE_STREAMING)
        get_possible_mapreduce_config_from.assert_called_once_with(
            'plugins/cdh/v5_9_0/resources/mapred-site.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch(
        'sahara_plugin_cdh.plugins.cdh.confighints_helper.'
        'get_possible_pig_config_from',
        return_value={})
    def test_get_possible_job_config_pig(self,
                                         get_possible_pig_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_PIG)
        get_possible_pig_config_from.assert_called_once_with(
            'plugins/cdh/v5_9_0/resources/mapred-site.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara_plugin_cdh.plugins.cdh.v5_9_0.edp_engine.'
                'EdpOozieEngine')
    def test_get_possible_job_config_shell(self, BaseCDHEdpOozieEngine):
        expected_config = {'job_config': {}}
        BaseCDHEdpOozieEngine.get_possible_job_config.return_value = (
            expected_config)
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_SHELL)
        BaseCDHEdpOozieEngine.get_possible_job_config.assert_called_once_with(
            edp.JOB_TYPE_SHELL)
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara.plugins.utils.get_config_value_or_default')
    @mock.patch('sahara.plugins.utils.get_instance')
    @mock.patch('sahara.plugins.edp.PluginsSparkJobEngine.'
                'validate_job_execution')
    def test_spark_engine_validate_job_execution(self,
                                                 validate_job_execution,
                                                 get_instance,
                                                 get_config_value_or_default):
        # version unsupported
        cluster = get_cluster(version='5.4.0')
        eng = edp_engine.EdpSparkEngine(cluster)
        self.assertRaises(pl_ex.PluginInvalidDataException,
                          eng.validate_job_execution, cluster,
                          mock.Mock(), mock.Mock())

        # none yarn history server
        cluster = get_cluster()
        eng = edp_engine.EdpSparkEngine(cluster)
        self.assertRaises(pl_ex.InvalidComponentCountException,
                          eng.validate_job_execution, cluster,
                          mock.Mock(), mock.Mock())

        # valid
        cluster = get_cluster()
        yarn_history_node_group = mock.Mock()
        yarn_history_node_group.node_processes = ['SPARK_YARN_HISTORY_SERVER']
        yarn_history_node_group.count = 1
        list.append(cluster.node_groups, yarn_history_node_group)
        eng = edp_engine.EdpSparkEngine(cluster)
        eng.validate_job_execution(cluster, mock.Mock(), mock.Mock())



# Copyright (c) 2017 Massachusetts Open Cloud
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cm_api.api_client import ApiResource

cm_host = "localhost"
api = ApiResource(cm_host, username="admin", password="admin")  # nosec

c = api.get_all_clusters()[0]
services = c.get_all_services()


def process_service(service):
    service_name = service.name
    if service_name == "spark_on_yarn":
        service_name = "spark"
    for role_cfgs in service.get_all_role_config_groups():
        role_cm_cfg = role_cfgs.get_config(view='full')
        role_cfg = parse_config(role_cm_cfg)
        role_name = role_cfgs.roleType.lower()
        write_cfg(role_cfg, '%s-%s.json' % (service_name, role_name))

    service_cm_cfg = service.get_config(view='full')[0]
    service_cfg = parse_config(service_cm_cfg)
    write_cfg(service_cfg, '%s-service.json' % service_name)


def parse_config(config):
    cfg = []
    for name, value in config.items():
        p = {
            'name': value.name,
            'value': value.default,
            'display_name': value.displayName,
            'desc': value.description
        }
        cfg.append(p)

    return cfg


def write_cfg(cfg, file_name):
    to_write = __import__('json').dumps(cfg, sort_keys=True, indent=4,
                                        separators=(',', ': '))

    with open(file_name, 'w') as f:
        f.write(to_write)


if __name__ == '__main__':
    for service in services:
        process_service(service)



# Copyright (c) 2015 ISPRAS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools
from unittest import mock

from sahara.plugins import base as pb
from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import edp
from sahara.plugins import exceptions as ex
from sahara_plugin_cdh.tests.unit import base


class SparkPluginTest(base.SaharaWithDbTestCase):
    def setUp(self):
        super(SparkPluginTest, self).setUp()
        self.override_config("plugins", ["cdh"])
        pb.setup_plugins()

    def test_plugin_edp_engine_no_spark(self):
        cluster_dict = {
            'name': 'cluster',
            'plugin_name': 'cdh',
            'hadoop_version': '5.7.0',
            'default_image_id': 'image'}

        job = mock.Mock()
        job.type = edp.JOB_TYPE_SPARK

        cluster = conductor.cluster_create(context.ctx(), cluster_dict)
        plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
        edp_engine = plugin.get_edp_engine(cluster, edp.JOB_TYPE_SPARK)
        with testtools.ExpectedException(
                ex.InvalidComponentCountException,
                value_re="Hadoop cluster should contain 1 "
                         "SPARK_YARN_HISTORY_SERVER component\(s\). Actual "
                         "SPARK_YARN_HISTORY_SERVER count is 0\nError ID: .*"):

            edp_engine.validate_job_execution(cluster, job, mock.Mock())

    def test_plugin_edp_engine(self):
        cluster_dict = {
            'name': 'cluster',
            'plugin_name': 'cdh',
            'hadoop_version': '5.7.0',
            'default_image_id': 'image'}

        cluster = conductor.cluster_create(context.ctx(), cluster_dict)
        plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
        self.assertIsInstance(
            plugin.get_edp_engine(cluster, edp.JOB_TYPE_SPARK),
            edp.PluginsSparkJobEngine)



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from sahara.plugins import provisioning as p
from sahara.plugins import utils


class ConfigHelper(object):
    path_to_config = ''

    CDH5_REPO_URL = p.Config(
        'CDH5 repo list URL', 'general', 'cluster', priority=1,
        default_value="")

    CDH5_REPO_KEY_URL = p.Config(
        'CDH5 repo key URL (for debian-based only)', 'general', 'cluster',
        priority=1, default_value="")

    CM5_REPO_URL = p.Config(
        'CM5 repo list URL', 'general', 'cluster', priority=1,
        default_value="")

    CM5_REPO_KEY_URL = p.Config(
        'CM5 repo key URL (for debian-based only)', 'general', 'cluster',
        priority=1, default_value="")

    ENABLE_HBASE_COMMON_LIB = p.Config(
        'Enable HBase Common Lib', 'general', 'cluster', config_type='bool',
        priority=1, default_value=True)

    ENABLE_SWIFT = p.Config(
        'Enable Swift', 'general', 'cluster',
        config_type='bool', priority=1, default_value=True)

    DEFAULT_SWIFT_LIB_URL = (
        'https://repository.cloudera.com/artifactory/repo/org'
        '/apache/hadoop/hadoop-openstack/2.6.0-cdh5.5.0'
        '/hadoop-openstack-2.6.0-cdh5.5.0.jar')

    SWIFT_LIB_URL = p.Config(
        'Hadoop OpenStack library URL', 'general', 'cluster', priority=1,
        default_value=DEFAULT_SWIFT_LIB_URL,
        description=("Library that adds Swift support to CDH. The file"
                     " will be downloaded by VMs."))

    DEFAULT_EXTJS_LIB_URL = (
        'https://tarballs.openstack.org/sahara-extra/dist/common-artifacts/'
        'ext-2.2.zip')

    EXTJS_LIB_URL = p.Config(
        "ExtJS library URL", 'general', 'cluster', priority=1,
        default_value=DEFAULT_EXTJS_LIB_URL,
        description=("Ext 2.2 library is required for Oozie Web Console. "
                     "The file will be downloaded by VMs with oozie."))

    _default_executor_classpath = ":".join(
        ['/usr/lib/hadoop/lib/jackson-core-asl-1.8.8.jar',
         '/usr/lib/hadoop-mapreduce/hadoop-openstack.jar'])

    EXECUTOR_EXTRA_CLASSPATH = p.Config(
        'Executor extra classpath', 'Spark', 'cluster', priority=2,
        default_value=_default_executor_classpath,
        description='Value for spark.executor.extraClassPath in '
                    'spark-defaults.conf (default: %s)'
                    % _default_executor_classpath)

    KMS_REPO_URL = p.Config(
        'KMS repo list URL', 'general', 'cluster', priority=1,
        default_value="")

    KMS_REPO_KEY_URL = p.Config(
        'KMS repo key URL (for debian-based only)', 'general',
        'cluster',
        priority=1, default_value="")

    REQUIRE_ANTI_AFFINITY = p.Config(
        'Require Anti Affinity', 'general', 'cluster',
        config_type='bool', priority=2, default_value=True)

    AWAIT_AGENTS_TIMEOUT = p.Config(
        'Await Cloudera agents timeout', 'general', 'cluster',
        config_type='int', priority=1, default_value=300, is_optional=True,
        description="Timeout for Cloudera agents connecting to"
                    " Cloudera Manager, in seconds")

    AWAIT_MANAGER_STARTING_TIMEOUT = p.Config(
        'Timeout for Cloudera Manager starting', 'general', 'cluster',
        config_type='int', priority=1, default_value=300, is_optional=True,
        description='Timeout for Cloudera Manager starting, in seconds')

    def __new__(cls):
        # make it a singleton
        if not hasattr(cls, '_instance'):
            cls._instance = super(ConfigHelper, cls).__new__(cls)
            setattr(cls, '__init__', cls.decorate_init(cls.__init__))
        return cls._instance

    @classmethod
    def decorate_init(cls, f):
        """decorate __init__ to prevent multiple calling."""
        def wrap(*args, **kwargs):
            if not hasattr(cls, '_init'):
                f(*args, **kwargs)
                cls._init = True
        return wrap

    def __init__(self):
        self.ng_plugin_configs = []
        self.priority_one_confs = {}

    def _load_json(self, path_to_file):
        data = utils.get_file_text(path_to_file, 'sahara_plugin_cdh')
        return json.loads(data)

    def _init_ng_configs(self, confs, app_target, scope):
        prepare_value = lambda x: x.replace('\n', ' ') if x else ""
        cfgs = []
        for cfg in confs:
            priority = 1 if cfg['name'] in self.priority_one_confs else 2
            c = p.Config(cfg['name'], app_target, scope, priority=priority,
                         default_value=prepare_value(cfg['value']),
                         description=cfg['desc'], is_optional=True)
            cfgs.append(c)

        return cfgs

    def _init_all_ng_plugin_configs(self):
        self.hdfs_confs = self._load_and_init_configs(
            'hdfs-service.json', 'HDFS', 'cluster')
        self.namenode_confs = self._load_and_init_configs(
            'hdfs-namenode.json', 'NAMENODE', 'node')
        self.datanode_confs = self._load_and_init_configs(
            'hdfs-datanode.json', 'DATANODE', 'node')
        self.secnamenode_confs = self._load_and_init_configs(
            'hdfs-secondarynamenode.json', 'SECONDARYNAMENODE', 'node')
        self.hdfs_gateway_confs = self._load_and_init_configs(
            'hdfs-gateway.json', 'HDFS_GATEWAY', 'node')
        self.journalnode_confs = self._load_and_init_configs(
            'hdfs-journalnode.json', 'JOURNALNODE', 'node')

        self.yarn_confs = self._load_and_init_configs(
            'yarn-service.json', 'YARN', 'cluster')
        self.resourcemanager_confs = self._load_and_init_configs(
            'yarn-resourcemanager.json', 'RESOURCEMANAGER', 'node')
        self.nodemanager_confs = self._load_and_init_configs(
            'yarn-nodemanager.json', 'NODEMANAGER', 'node')
        self.jobhistory_confs = self._load_and_init_configs(
            'yarn-jobhistory.json', 'JOBHISTORY', 'node')
        self.yarn_gateway_conf = self._load_and_init_configs(
            'yarn-gateway.json', 'YARN_GATEWAY', 'node')

        self.oozie_service_confs = self._load_and_init_configs(
            'oozie-service.json', 'OOZIE', 'cluster')
        self.oozie_role_confs = self._load_and_init_configs(
            'oozie-oozie_server.json', 'OOZIE', 'node')

        self.hive_service_confs = self._load_and_init_configs(
            'hive-service.json', 'HIVE', 'cluster')
        self.hive_metastore_confs = self._load_and_init_configs(
            'hive-hivemetastore.json', 'HIVEMETASTORE', 'node')
        self.hive_hiveserver_confs = self._load_and_init_configs(
            'hive-hiveserver2.json', 'HIVESERVER', 'node')
        self.hive_webhcat_confs = self._load_and_init_configs(
            'hive-webhcat.json', 'WEBHCAT', 'node')

        self.hue_service_confs = self._load_and_init_configs(
            'hue-service.json', 'HUE', 'cluster')
        self.hue_role_confs = self._load_and_init_configs(
            'hue-hue_server.json', 'HUE', 'node')

        self.spark_service_confs = self._load_and_init_configs(
            'spark-service.json', 'SPARK_ON_YARN', 'cluster')
        self.spark_role_confs = self._load_and_init_configs(
            'spark-spark_yarn_history_server.json', 'SPARK_ON_YARN', 'node')

        self.zookeeper_server_confs = self._load_and_init_configs(
            'zookeeper-service.json', 'ZOOKEEPER', 'cluster')
        self.zookeeper_service_confs = self._load_and_init_configs(
            'zookeeper-server.json', 'ZOOKEEPER', 'node')

        self.hbase_confs = self._load_and_init_configs(
            'hbase-service.json', 'HBASE', 'cluster')
        self.master_confs = self._load_and_init_configs(
            'hbase-master.json', 'MASTER', 'node')
        self.regionserver_confs = self._load_and_init_configs(
            'hbase-regionserver.json', 'REGIONSERVER', 'node')

        self.flume_service_confs = self._load_and_init_configs(
            'flume-service.json', 'FLUME', 'cluster')
        self.flume_agent_confs = self._load_and_init_configs(
            'flume-agent.json', 'FLUME', 'node')

        self.sentry_service_confs = self._load_and_init_configs(
            'sentry-service.json', 'SENTRY', 'cluster')
        self.sentry_server_confs = self._load_and_init_configs(
            'sentry-sentry_server.json', 'SENTRY', 'node')

        self.solr_service_confs = self._load_and_init_configs(
            'solr-service.json', 'SOLR', 'cluster')
        self.solr_server_confs = self._load_and_init_configs(
            'solr-solr_server.json', 'SOLR', 'node')

        self.sqoop_service_confs = self._load_and_init_configs(
            'sqoop-service.json', 'SQOOP', 'cluster')
        self.sqoop_server_confs = self._load_and_init_configs(
            'sqoop-sqoop_server.json', 'SQOOP', 'node')

        self.ks_indexer_service_confs = self._load_and_init_configs(
            'ks_indexer-service.json', 'KS_INDEXER', 'cluster')
        self.ks_indexer_role_confs = self._load_and_init_configs(
            'ks_indexer-hbase_indexer.json', 'KS_INDEXER', 'node')

        self.impala_service_confs = self._load_and_init_configs(
            'impala-service.json', 'IMPALA', 'cluster')
        self.impala_catalogserver_confs = self._load_and_init_configs(
            'impala-catalogserver.json', 'CATALOGSERVER', 'node')
        self.impala_impalad_confs = self._load_and_init_configs(
            'impala-impalad.json', 'IMPALAD', 'node')
        self.impala_statestore_confs = self._load_and_init_configs(
            'impala-statestore.json', 'STATESTORE', 'node')

        self.kms_service_confs = self._load_and_init_configs(
            'kms-service.json', 'KMS', 'cluster')
        self.kms_kms_confs = self._load_and_init_configs(
            'kms-kms.json', 'KMS', 'node')

        self.kafka_service = self._load_and_init_configs(
            'kafka-service.json', 'KAFKA', 'cluster')
        self.kafka_kafka_broker = self._load_and_init_configs(
            'kafka-kafka_broker.json', 'KAFKA', 'node')
        self.kafka_kafka_mirror_maker = self._load_and_init_configs(
            'kafka-kafka_mirror_maker.json', 'KAFKA', 'node')

    def _load_and_init_configs(self, filename, app_target, scope):
        confs = self._load_json(self.path_to_config + filename)
        cfgs = self._init_ng_configs(confs, app_target, scope)
        self.ng_plugin_configs += cfgs

        return cfgs

    def _get_ng_plugin_configs(self):
        return self.ng_plugin_configs

    def _get_cluster_plugin_configs(self):
        return [self.CDH5_REPO_URL, self.CDH5_REPO_KEY_URL, self.CM5_REPO_URL,
                self.CM5_REPO_KEY_URL, self.ENABLE_SWIFT, self.SWIFT_LIB_URL,
                self.ENABLE_HBASE_COMMON_LIB, self.EXTJS_LIB_URL,
                self.AWAIT_MANAGER_STARTING_TIMEOUT, self.AWAIT_AGENTS_TIMEOUT,
                self.EXECUTOR_EXTRA_CLASSPATH, self.KMS_REPO_URL,
                self.KMS_REPO_KEY_URL, self.REQUIRE_ANTI_AFFINITY]

    def get_plugin_configs(self):
        cluster_wide = self._get_cluster_plugin_configs()
        ng_wide = self._get_ng_plugin_configs()
        return cluster_wide + ng_wide

    def _get_config_value(self, cluster, key):
        return cluster.cluster_configs.get(
            'general', {}).get(key.name, key.default_value)

    def get_cdh5_repo_url(self, cluster):
        return self._get_config_value(cluster, self.CDH5_REPO_URL)

    def get_cdh5_key_url(self, cluster):
        return self._get_config_value(cluster, self.CDH5_REPO_KEY_URL)

    def get_cm5_repo_url(self, cluster):
        return self._get_config_value(cluster, self.CM5_REPO_URL)

    def get_cm5_key_url(self, cluster):
        return self._get_config_value(cluster, self.CM5_REPO_KEY_URL)

    def is_swift_enabled(self, cluster):
        return self._get_config_value(cluster, self.ENABLE_SWIFT)

    def is_hbase_common_lib_enabled(self, cluster):
        return self._get_config_value(cluster,
                                      self.ENABLE_HBASE_COMMON_LIB)

    def is_keytrustee_available(self):
        return True

    def get_swift_lib_url(self, cluster):
        return self._get_config_value(cluster, self.SWIFT_LIB_URL)

    def get_extjs_lib_url(self, cluster):
        return self._get_config_value(cluster, self.EXTJS_LIB_URL)

    def get_kms_key_url(self, cluster):
        return self._get_config_value(cluster, self.KMS_REPO_KEY_URL)

    def get_required_anti_affinity(self, cluster):
        return self._get_config_value(cluster, self.REQUIRE_ANTI_AFFINITY)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import utils
from sahara_plugin_cdh.i18n import _
from sahara_plugin_cdh.tests.unit import base as b
from sahara_plugin_cdh.tests.unit.plugins.cdh import utils as ctu

CONFIGURATION_SCHEMA = {
    'node_configs': {
        'yarn.scheduler.minimum-allocation-mb': (
            'RESOURCEMANAGER', 'yarn_scheduler_minimum_allocation_mb'),
        'mapreduce.reduce.memory.mb': (
            'YARN_GATEWAY', 'mapreduce_reduce_memory_mb'),
        'mapreduce.map.memory.mb': (
            'YARN_GATEWAY', 'mapreduce_map_memory_mb',),
        'yarn.scheduler.maximum-allocation-mb': (
            'RESOURCEMANAGER', 'yarn_scheduler_maximum_allocation_mb'),
        'yarn.app.mapreduce.am.command-opts': (
            'YARN_GATEWAY', 'yarn_app_mapreduce_am_command_opts'),
        'yarn.nodemanager.resource.memory-mb': (
            'NODEMANAGER', 'yarn_nodemanager_resource_memory_mb'),
        'mapreduce.task.io.sort.mb': (
            'YARN_GATEWAY', 'io_sort_mb'),
        'mapreduce.map.java.opts': (
            'YARN_GATEWAY', 'mapreduce_map_java_opts'),
        'mapreduce.reduce.java.opts': (
            'YARN_GATEWAY', 'mapreduce_reduce_java_opts'),
        'yarn.app.mapreduce.am.resource.mb': (
            'YARN_GATEWAY', 'yarn_app_mapreduce_am_resource_mb')
    },
    'cluster_configs': {
        'dfs.replication': ('HDFS', 'dfs_replication')
    }
}


def get_concrete_cluster():
    cluster = ctu.get_fake_cluster()

    # add configs to cluster
    configs = {"SQOOP": {}, "HUE": {}, "general": {}, "KMS": {}, "HIVE": {},
               "SOLR": {}, "FLUME": {}, "HDFS": {"dfs_replication": 1},
               "KS_INDEXER": {}, "SPARK_ON_YARN": {}, "SENTRY": {}, "YARN": {},
               "ZOOKEEPER": {}, "OOZIE": {}, "HBASE": {}, "IMPALA": {}}
    # cluster is immutable, a work around
    dict.__setitem__(cluster, "cluster_configs", configs)

    # add fake remotes to instances
    instances = [i for ng in cluster.node_groups for i in ng.instances]
    for i in instances:
        object.__setattr__(i, 'remote', mock.MagicMock())

    # add cluster_id to each node group
    for ng in cluster.node_groups:
        dict.__setitem__(ng, "cluster_id", ng.cluster.id)

    # add extra config
    dict.__setitem__(cluster, "extra", {})
    return cluster


def get_fake_worker_instances():
    ng = get_concrete_cluster().node_groups[2]
    return ng.instances


class TestPluginUtils(b.SaharaTestCase):

    def setUp(self):
        super(TestPluginUtils, self).setUp()
        self.plug_utils = None

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.plugin_utils.'
                'CDHPluginAutoConfigsProvider')
    def test_recommend_configs(self, provider, log_cfg):
        fake_plugin_utils = mock.Mock()
        fake_cluster = mock.Mock()
        self.plug_utils.recommend_configs(
            fake_cluster, fake_plugin_utils, False)
        self.assertEqual([mock.call(CONFIGURATION_SCHEMA,
                                    fake_plugin_utils,
                                    fake_cluster,
                                    False)],
                         provider.call_args_list)

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.install_packages')
    def test_install_packages(self, install_packages, log_cfg):
        packages = mock.Mock()
        instances = get_fake_worker_instances()
        self.plug_utils.install_packages(instances, packages)

        calls = [mock.call(i.remote().__enter__(), packages)
                 for i in instances]

        install_packages.assert_has_calls(calls, any_order=False)

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.start_agent')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.configure_agent')
    def test_start_cloudera_agents(self, configure_agent,
                                   start_agent, log_cfg):
        instances = get_fake_worker_instances()

        self.plug_utils.start_cloudera_agents(instances)

        cfg_calls = [mock.call(i.remote().__enter__(), 'manager_inst')
                     for i in instances]
        start_calls = [mock.call(i.remote().__enter__()) for i in instances]

        configure_agent.assert_has_calls(cfg_calls, any_order=False)
        start_agent.assert_has_calls(start_calls, any_order=False)

    @mock.patch('sahara.config.CONF.disable_event_log')
    def test_put_hive_hdfs_xml(self, log_cfg):
        cluster = get_concrete_cluster()
        hive_server = cluster.node_groups[1].instances[0]
        self.plug_utils.put_hive_hdfs_xml(cluster)
        with hive_server.remote() as r:
            calls = [mock.call('sudo su - -c "hadoop fs -mkdir -p'
                               ' /user/hdfs/conf" hdfs'),
                     mock.call('sudo su - -c "hadoop fs -put'
                               ' /etc/hive/conf/hive-site.xml'
                               ' /user/hdfs/conf/hive-site.xml" hdfs')]
            r.execute_command.assert_has_calls(calls, any_order=False)

    @mock.patch('sahara.config.CONF.disable_event_log')
    def test_configure_swift(self, log_cfg):

        cluster = get_concrete_cluster()
        cluster.cluster_configs['general']['Enable Swift'] = True
        instances = [i for ng in cluster.node_groups for i in ng.instances]

        self.plug_utils.configure_swift(cluster)
        for i in instances:
            with i.remote() as r:
                cmd = r'ls /usr/lib/hadoop-mapreduce/hadoop-openstack.jar'
                # use any_call because the execute_command has a call:
                # call().__getitem__().__ne__(0) during the method
                r.execute_command.assert_any_call(cmd,
                                                  raise_when_error=False)
                cmd = (r'sudo curl %s'
                       r' -o /usr/lib/hadoop-mapreduce/hadoop-openstack.jar')
                cmd = cmd % self.plug_utils.c_helper.get_swift_lib_url(cluster)
                r.execute_command.call
                r.execute_command.assert_any_call(cmd)

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('uuid.uuid4')
    @mock.patch('sahara.plugins.conductor.cluster_update')
    @mock.patch('sahara.plugins.conductor.cluster_get')
    @mock.patch('castellan.key_manager.API')
    def test_configure_hive(self, keymanager, cluster_get,
                            cluster_update, uuid4, log_cfg):
        cluster = get_concrete_cluster()
        manager = cluster.node_groups[0].instances[0]
        cluster_get.return_value = cluster
        db_password = 'a8f2939f-ff9f-4659-a333-abc012ee9b2d'
        uuid4.return_value = db_password
        create_db_script = utils.get_file_text(
            'plugins/cdh/db_resources/create_hive_db.sql'
                                .format(version=self.version),
            'sahara_plugin_cdh')
        create_db_script = (create_db_script % db_password).encode('utf-8')

        self.plug_utils.configure_hive(cluster)

        with manager.remote() as r:
            cmd_exe_sql = ('PGPASSWORD=$(sudo head -1'
                           ' /var/lib/cloudera-scm-server-db/data/'
                           'generated_password.txt) psql'
                           ' -U cloudera-scm -h localhost -p 7432 -d scm -f'
                           ' script_to_exec.sql')
            cmd_clean = 'rm script_to_exec.sql'
            self.assertEqual(create_db_script, r.write_file_to.call_args[0][1])
            r.execute_command.assert_has_calls([mock.call(cmd_exe_sql),
                                                mock.call(cmd_clean)])

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.is_ubuntu_os')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.is_centos_os')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.update_repository')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.add_apt_key')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.'
                'write_ubuntu_repository')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.'
                'write_centos_repository')
    def test_configure_os(self, write_centos_repository,
                          write_ubuntu_repository, add_apt_key,
                          update_repository, is_centos_os,
                          is_ubuntu_os, log_cfg):
        cluster = get_concrete_cluster()
        ubuntu_instance = cluster.node_groups[2].instances[0]
        centos_instance = cluster.node_groups[2].instances[1]
        instances = [ubuntu_instance, centos_instance]

        is_ubuntu_os.side_effect = \
            lambda r: r is ubuntu_instance.remote().__enter__()
        is_centos_os.side_effect = \
            lambda r: r is centos_instance.remote().__enter__()

        self.plug_utils.configure_os(instances)

        with ubuntu_instance.remote() as r:
            write_ubuntu_repository.assert_has_calls(
                [mock.call(r, self.plug_utils.c_helper.CDH5_UBUNTU_REPO,
                           'cdh'),
                 mock.call(r, self.plug_utils.c_helper.CM5_UBUNTU_REPO,
                           'cm')],
                any_order=False)
            add_apt_key.assert_has_calls(
                [mock.call(r,
                           self.plug_utils.c_helper.
                           DEFAULT_CDH5_UBUNTU_REPO_KEY_URL),
                 mock.call(r,
                           self.plug_utils.c_helper.
                           DEFAULT_CM5_UBUNTU_REPO_KEY_URL)],
                any_order=False)
            update_repository.assert_any_call(r)

        with centos_instance.remote() as r:
            write_centos_repository.assert_has_calls(
                [mock.call(r, self.plug_utils.c_helper.CDH5_CENTOS_REPO,
                           'cdh'),
                 mock.call(r, self.plug_utils.c_helper.CM5_CENTOS_REPO,
                           'cm')],
                any_order=False)
            update_repository.assert_any_call(r)

    @mock.patch('sahara.config.CONF.disable_event_log')
    def test_install_extjs(self, log_cfg):

        cluster = get_concrete_cluster()
        oozie_server = cluster.node_groups[1].instances[0]
        self.plug_utils.install_extjs(cluster)
        with oozie_server.remote() as r:
            calls = [mock.call('ls /var/lib/oozie/ext-2.2',
                               raise_when_error=False),
                     mock.call("curl -L -o '/var/lib/oozie/extjs.zip'"
                               " https://tarballs.openstack.org/sahara-extra/"
                               "dist/common-artifacts/ext-2.2.zip",
                               run_as_root=True),
                     mock.call('unzip /var/lib/oozie/extjs.zip'
                               ' -d /var/lib/oozie',
                               run_as_root=True)]
            r.execute_command.assert_has_calls(calls, any_order=True)

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('sahara.plugins.utils.plugin_option_poll')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.start_cloudera_db')
    @mock.patch('sahara_plugin_cdh.plugins.cdh.commands.start_manager')
    def test_start_cloudera_manager(self, start_manager, start_cloudera_db,
                                    plugin_option_poll, log_cfg):

        cluster = get_concrete_cluster()
        manager = cluster.node_groups[0].instances[0]

        self.plug_utils.start_cloudera_manager(cluster)
        with manager.remote() as r:
            start_manager.assert_called_once_with(r)
            start_cloudera_db.assert_called_once_with(r)

        call = [cluster,
                self.plug_utils._check_cloudera_manager_started,
                self.plug_utils.c_helper.AWAIT_MANAGER_STARTING_TIMEOUT,
                _("Await starting Cloudera Manager"),
                2, {'manager': manager}]
        plugin_option_poll.assert_called_once_with(*call)

    def test_get_config_value(self):
        cluster = get_concrete_cluster()
        dfs_replication = self.plug_utils.get_config_value(
            'HDFS', 'dfs_replication', cluster)
        self.assertEqual(1, dfs_replication)
        dfs_replication_default = self.plug_utils.get_config_value(
            'HDFS', 'dfs_replication')
        self.assertEqual(3, dfs_replication_default)


class TestPluginUtilsHigherThanV5(TestPluginUtils):

    @mock.patch('sahara.config.CONF.disable_event_log')
    @mock.patch('uuid.uuid4')
    @mock.patch('sahara.plugins.conductor.cluster_update')
    @mock.patch('sahara.plugins.conductor.cluster_get')
    @mock.patch('castellan.key_manager.API')
    def test_configure_sentry(self, keymanager, cluster_get,
                              cluster_update, uuid4, cfg_log):
        cluster = get_concrete_cluster()
        manager = cluster.node_groups[0].instances[0]
        cluster_get.return_value = cluster
        db_password = 'a8f2939f-ff9f-4659-a333-abc012ee9b2d'
        uuid4.return_value = db_password
        create_db_script = utils.get_file_text(
            'plugins/cdh/db_resources/create_sentry_db.sql'
                                .format(version=self.version),
            'sahara_plugin_cdh')
        create_db_script = (create_db_script % db_password).encode('utf-8')

        self.plug_utils.configure_sentry(cluster)

        with manager.remote() as r:
            cmd_exe_sql = ('PGPASSWORD=$(sudo head -1'
                           ' /var/lib/cloudera-scm-server-db/data/'
                           'generated_password.txt) psql'
                           ' -U cloudera-scm -h localhost -p 7432 -d scm -f'
                           ' script_to_exec.sql')
            cmd_clean = 'rm script_to_exec.sql'
            self.assertEqual(create_db_script, r.write_file_to.call_args[0][1])
            r.execute_command.assert_has_calls([mock.call(cmd_exe_sql),
                                                mock.call(cmd_clean)])



# Copyright (c) 2015 Intel Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_utils import uuidutils

from sahara.plugins import castellan_utils as key_manager
from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import utils

CM_PASSWORD = 'cm_password'
HIVE_DB_PASSWORD = 'hive_db_password'
SENTRY_DB_PASSWORD = 'sentry_db_password'


def delete_password_from_keymanager(cluster, pwname):
    """delete the named password from the key manager

    This function will lookup the named password in the cluster entry
    and delete it from the key manager.

    :param cluster: The cluster record containing the password
    :param pwname: The name associated with the password
    """
    ctx = context.ctx()
    cluster = conductor.cluster_get(ctx, cluster.id)
    key_id = cluster.extra.get(pwname) if cluster.extra else None
    if key_id is not None:
        key_manager.delete_key(key_id, ctx)


def delete_passwords_from_keymanager(cluster):
    """delete all passwords associated with a cluster

    This function will remove all passwords stored in a cluster database
    entry from the key manager.

    :param cluster: The cluster record containing the passwords
    """
    delete_password_from_keymanager(cluster, CM_PASSWORD)
    delete_password_from_keymanager(cluster, HIVE_DB_PASSWORD)
    delete_password_from_keymanager(cluster, SENTRY_DB_PASSWORD)


def get_password_from_db(cluster, pwname):
    """return a password for the named entry

    This function will return, or create and return, a password for the
    named entry. It will store the password in the key manager and use
    the ID in the database entry.

    :param cluster: The cluster record containing the password
    :param pwname: The entry name associated with the password
    :returns: The cleartext password
    """
    ctx = context.ctx()
    cluster = conductor.cluster_get(ctx, cluster.id)
    passwd = cluster.extra.get(pwname) if cluster.extra else None
    if passwd:
        return key_manager.get_secret(passwd, ctx)

    passwd = uuidutils.generate_uuid()
    extra = cluster.extra.to_dict() if cluster.extra else {}
    extra[pwname] = key_manager.store_secret(passwd, ctx)
    conductor.cluster_update(ctx, cluster, {'extra': extra})
    return passwd


def get_cm_password(cluster):
    return get_password_from_db(cluster, CM_PASSWORD)


def remote_execute_db_script(remote, script_content):
    script_name = 'script_to_exec.sql'
    remote.write_file_to(script_name, script_content)

    psql_cmd = ('PGPASSWORD=$(sudo head -1 /var/lib/cloudera-scm-server-db'
                '/data/generated_password.txt) psql -U cloudera-scm '
                '-h localhost -p 7432 -d scm -f %s') % script_name
    remote.execute_command(psql_cmd)
    remote.execute_command('rm %s' % script_name)


def get_hive_db_password(cluster):
    return get_password_from_db(cluster, 'hive_db_password')


def get_sentry_db_password(cluster):
    return get_password_from_db(cluster, 'sentry_db_password')


def create_hive_database(cluster, remote):
    db_password = get_hive_db_password(cluster)
    create_db_script = utils.try_get_file_text(
        'plugins/cdh/db_resources/create_hive_db.sql', 'sahara_plugin_cdh')
    create_db_script = create_db_script % db_password.encode('utf-8')
    remote_execute_db_script(remote, create_db_script)


def create_sentry_database(cluster, remote):
    db_password = get_sentry_db_password(cluster)
    create_db_script = utils.try_get_file_text(
        'plugins/cdh/db_resources/create_sentry_db.sql', 'sahara_plugin_cdh')
    create_db_script = create_db_script % db_password.encode('utf-8')
    remote_execute_db_script(remote, create_db_script)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara_plugin_cdh.plugins.cdh import plugin_utils as pu
from sahara_plugin_cdh.tests.unit import base
from sahara_plugin_cdh.tests.unit.plugins.cdh import utils as tu


PU = pu.AbstractPluginUtils()


class UtilsTestCase(base.SaharaTestCase):
    def test_get_manager(self):
        cluster = tu.get_fake_cluster()
        inst = PU.get_manager(cluster)
        self.assertEqual('id1', inst.instance_id)

    def test_get_namenode(self):
        cluster = tu.get_fake_cluster()
        inst = PU.get_namenode(cluster)
        self.assertEqual('id2', inst.instance_id)

    def test_get_secondarynamenode(self):
        cluster = tu.get_fake_cluster()
        inst = PU.get_secondarynamenode(cluster)
        self.assertEqual('id2', inst.instance_id)

    def test_get_resourcemanager(self):
        cluster = tu.get_fake_cluster()
        inst = PU.get_resourcemanager(cluster)
        self.assertEqual('id2', inst.instance_id)

    def test_get_datanodes(self):
        cluster = tu.get_fake_cluster()
        dns = PU.get_datanodes(cluster)
        ids = [dn.instance_id for dn in dns]
        self.assertEqual(sorted(['id00', 'id01', 'id02']), sorted(ids))

    def test_get_nodemanagers(self):
        cluster = tu.get_fake_cluster()
        nms = PU.get_nodemanagers(cluster)
        ids = [nm.instance_id for nm in nms]
        self.assertEqual(sorted(['id00', 'id01', 'id02']), sorted(ids))

    def test_get_historyserver(self):
        cluster = tu.get_fake_cluster()
        inst = PU.get_historyserver(cluster)
        self.assertEqual('id2', inst.instance_id)

    def test_get_oozie(self):
        cluster = tu.get_fake_cluster()
        inst = PU.get_oozie(cluster)
        self.assertEqual('id2', inst.instance_id)



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

from oslo_log import log as logging

from sahara.plugins import health_check_base
from sahara_plugin_cdh.i18n import _

LOG = logging.getLogger(__name__)


class HealthStatusProvider(object):
    def __init__(self, cluster, cloudera_tools):
        self.cluster = cluster
        self.cloudera_tools = cloudera_tools
        self._data = None
        self._cluster_services = None
        self._exception_store = None
        self.get_health_status()

    def get_cluster_services(self):
        return self._cluster_services

    def is_cloudera_active(self):
        if self._exception_store:
            raise health_check_base.RedHealthError(self._exception_store)
        return _("Cloudera Manager is Active")

    def get_cloudera_health(self):
        cu = self.cloudera_tools
        api = cu.get_api_client(self.cluster)
        return api.get_service_health_status(self.cluster.name)

    def get_important_services(self):
        # will be overridable in future
        cu = self.cloudera_tools
        return [
            cu.HDFS_SERVICE_NAME,
            cu.YARN_SERVICE_NAME,
            cu.OOZIE_SERVICE_NAME
        ]

    def get_health_status(self, service=None):
        if self._data is not None:
            return self._data.get(service, []) if service else self._data
        self._data = {}
        self._cluster_services = []
        try:
            # all data already grouped by services
            self._data = self.get_cloudera_health()
            self._cluster_services = self._data.keys()
        except Exception as e:
            msg = _("Can't get response from Cloudera "
                    "Manager")
            LOG.exception(msg)
            self._exception_store = _(
                "%(problem)s, reason: %(reason)s") % {
                'problem': msg, 'reason': str(e)}


class ClouderaManagerHealthCheck(health_check_base.BasicHealthCheck):
    def __init__(self, cluster, provider):
        self.provider = provider
        super(ClouderaManagerHealthCheck, self).__init__(cluster)

    def get_health_check_name(self):
        return _("Cloudera Manager health check")

    def is_available(self):
        return self.cluster.plugin_name == 'cdh'

    def check_health(self):
        return self.provider.is_cloudera_active()


class ServiceHealthCheck(health_check_base.BasicHealthCheck):
    def __init__(self, cluster, provider, service):
        self.provider = provider
        self.service = service
        super(ServiceHealthCheck, self).__init__(cluster)

    def get_health_check_name(self):
        return _("CDH %s health check") % self.service

    def is_available(self):
        return self.cluster.plugin_name == 'cdh'

    def check_health(self):
        important_services = self.provider.get_important_services()
        observed_data = self.provider.get_health_status(self.service)
        imp_map = {'BAD': 'red', 'CONCERNING': 'yellow', 'GOOD': 'green'}
        summary = observed_data['summary']
        checks = observed_data.get('checks', [])
        failed_checks = []
        for check in checks:
            if check['summary'] != 'GOOD':
                failed_checks.append('%(name)s - %(summary)s state' % {
                    'name': check['name'], 'summary': check['summary']
                })
        additional_info = None
        if failed_checks:
            additional_info = _(
                "The following checks did not pass: %s") % ",".join(
                failed_checks)
        if self.service in important_services:
            overall = imp_map.get(summary, 'red')
        else:
            overall = 'green'
            if summary != 'GOOD':
                overall = 'yellow'
        msg = _("Cloudera Manager has responded that service is in "
                "the %s state") % summary
        if additional_info:
            msg = _("%(problem)s. %(description)s") % {
                'problem': msg, 'description': additional_info}
        if overall == 'red':
            raise health_check_base.RedHealthError(msg)
        elif overall == 'yellow':
            raise health_check_base.YellowHealthError(msg)
        return msg


def get_health_checks(cluster, cloudera_utils):
    provider = HealthStatusProvider(cluster, cloudera_utils)
    checks = [functools.partial(
        ClouderaManagerHealthCheck, provider=provider)]
    for service in provider.get_cluster_services():
        checks.append(functools.partial(
            ServiceHealthCheck, provider=provider, service=service))
    return checks



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import edp
from sahara_plugin_cdh.plugins.cdh import confighints_helper as ch_helper
from sahara_plugin_cdh.plugins.cdh import edp_engine
from sahara_plugin_cdh.plugins.cdh.v5_7_0 import cloudera_utils as cu


class EdpOozieEngine(edp_engine.EdpOozieEngine):

    def __init__(self, cluster):
        super(EdpOozieEngine, self).__init__(cluster)
        self.cloudera_utils = cu.ClouderaUtilsV570()

    @staticmethod
    def get_possible_job_config(job_type):
        if edp.compare_job_type(job_type, edp.JOB_TYPE_HIVE):
            return {'job_config': ch_helper.get_possible_hive_config_from(
                    'plugins/cdh/v5_7_0/resources/hive-site.xml')}
        if edp.compare_job_type(job_type,
                                edp.JOB_TYPE_MAPREDUCE,
                                edp.JOB_TYPE_MAPREDUCE_STREAMING):
            return {'job_config': ch_helper.get_possible_mapreduce_config_from(
                    'plugins/cdh/v5_7_0/resources/mapred-site.xml')}
        if edp.compare_job_type(job_type, edp.JOB_TYPE_PIG):
            return {'job_config': ch_helper.get_possible_pig_config_from(
                    'plugins/cdh/v5_7_0/resources/mapred-site.xml')}
        return edp.PluginsOozieJobEngine.get_possible_job_config(job_type)


class EdpSparkEngine(edp_engine.EdpSparkEngine):

    edp_base_version = "5.7.0"



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import exceptions as ex
from sahara_plugin_cdh.i18n import _


def _root(remote, cmd, **kwargs):
    return remote.execute_command(cmd, run_as_root=True, **kwargs)


def _get_os_distrib(remote):
    return remote.get_os_distrib()


def is_centos_os(remote):
    return _get_os_distrib(remote) == 'centos'


def is_ubuntu_os(remote):
    return _get_os_distrib(remote) == 'ubuntu'


def is_pre_installed_cdh(remote):
    code, out = remote.execute_command('ls /etc/init.d/cloudera-scm-server',
                                       raise_when_error=False)
    return code == 0


def start_cloudera_db(remote):
    _root(remote, 'service cloudera-scm-server-db start')
    # for Hive access
    hive_access_param = 'host metastore hive 0.0.0.0/0 md5'
    remote.append_to_file('/var/lib/cloudera-scm-server-db/data/pg_hba.conf',
                          hive_access_param, run_as_root=True)
    _root(remote, 'service cloudera-scm-server-db restart')


def start_manager(remote):
    _root(remote, 'service cloudera-scm-server start')


def configure_agent(remote, manager_address):
    remote.replace_remote_string('/etc/cloudera-scm-agent/config.ini',
                                 'server_host=.*',
                                 'server_host=%s' % manager_address)


def start_agent(remote):
    _root(remote, 'service cloudera-scm-agent start')


def install_packages(remote, packages, timeout=1800):
    distrib = _get_os_distrib(remote)
    if distrib == 'ubuntu':
        cmd = 'RUNLEVEL=1 apt-get install -y %s'
    elif distrib == 'centos':
        cmd = 'yum install -y %s'
    else:
        raise ex.HadoopProvisionError(
            _("OS on image is not supported by CDH plugin"))

    cmd = cmd % ' '.join(packages)
    _root(remote, cmd, timeout=timeout)


def update_repository(remote):
    if is_ubuntu_os(remote):
        _root(remote, 'apt-get update')
    if is_centos_os(remote):
        _root(remote, 'yum clean all')


def push_remote_file(remote, src, dst):
    cmd = 'curl %s -o %s' % (src, dst)
    _root(remote, cmd)


def add_ubuntu_repository(r, repo_list_url, repo_name):
    push_remote_file(r, repo_list_url,
                     '/etc/apt/sources.list.d/%s.list' % repo_name)


def write_ubuntu_repository(r, repo_content, repo_name):
    r.write_file_to('/etc/apt/sources.list.d/%s.list' % repo_name,
                    repo_content, run_as_root=True)


def add_apt_key(remote, key_url):
    cmd = 'wget -qO - %s | apt-key add -' % key_url
    _root(remote, cmd)


def add_centos_repository(r, repo_list_url, repo_name):
    push_remote_file(r, repo_list_url, '/etc/yum.repos.d/%s.repo' % repo_name)


def write_centos_repository(r, repo_content, repo_name):
    r.write_file_to('/etc/yum.repos.d/%s.repo' % repo_name,
                    repo_content, run_as_root=True)


def start_mysql_server(remote):
    _root(remote, 'service mysql start')



# Copyright (c) 2015 Intel Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara_plugin_cdh.plugins.cdh.v5_11_0 import plugin_utils as pu
from sahara_plugin_cdh.tests.unit.plugins.cdh import base_plugin_utils_test


class TestPluginUtilsV5110(base_plugin_utils_test.TestPluginUtilsHigherThanV5):

    def setUp(self):
        super(TestPluginUtilsV5110, self).setUp()
        self.plug_utils = pu.PluginUtilsV5110()
        self.version = "v5_11_0"



# Copyright (c) 2015 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from sahara_plugin_cdh.plugins.cdh.client import api_client

# -- cm config --

cm_address = 'localhost'
cm_port = 7180
cm_username = 'admin'
cm_password = 'admin'

hdfs_service_name = 'hdfs01'
yarn_service_name = 'yarn01'
oozie_service_name = 'oozie01'
hive_service_name = 'hive01'
hue_service_name = 'hue01'
spark_service_name = 'spark_on_yarn01'
zookeeper_service_name = 'zookeeper01'
hbase_service_name = 'hbase01'
flume_service_name = 'flume01'
sqoop_service_name = 'sqoop01'
solr_service_name = 'solr01'
ks_indexer_service_name = 'ks_indexer01'
impala_service_name = 'impala01'
sentry_service_name = 'sentry01'
kms_service_name = 'kms01'


def get_cm_api():
    return api_client.ApiResource(cm_address, server_port=cm_port,
                                  username=cm_username, password=cm_password)


def get_cluster(api):
    return api.get_all_clusters()[0]


def process_service(service, service_name):
    for role_cfgs in service.get_all_role_config_groups():
        role_cm_cfg = role_cfgs.get_config(view='full')
        role_cfg = parse_config(role_cm_cfg)
        role_name = role_cfgs.roleType.lower()
        write_cfg(role_cfg, '%s-%s.json' % (service_name, role_name))

    service_cm_cfg = service.get_config(view='full')[0]
    service_cfg = parse_config(service_cm_cfg)
    write_cfg(service_cfg, '%s-service.json' % service_name)


def parse_config(config):
    cfg = []
    for name, value in config.items():
        p = {
            'name': value.name,
            'value': value.default,
            'display_name': value.displayName,
            'desc': value.description
        }
        cfg.append(p)

    return cfg


def write_cfg(cfg, file_name):
    to_write = json.dumps(cfg, sort_keys=True, indent=4,
                          separators=(',', ': '))

    with open(file_name, 'w') as f:
        f.write(to_write)


def main():
    client = get_cm_api()
    cluster = get_cluster(client)

    hdfs = cluster.get_service(hdfs_service_name)
    process_service(hdfs, 'hdfs')

    yarn = cluster.get_service(yarn_service_name)
    process_service(yarn, 'yarn')

    oozie = cluster.get_service(oozie_service_name)
    process_service(oozie, 'oozie')

    hive = cluster.get_service(hive_service_name)
    process_service(hive, 'hive')

    hue = cluster.get_service(hue_service_name)
    process_service(hue, 'hue')

    spark = cluster.get_service(spark_service_name)
    process_service(spark, 'spark')

    zookeeper = cluster.get_service(zookeeper_service_name)
    process_service(zookeeper, 'zookeeper')

    hbase = cluster.get_service(hbase_service_name)
    process_service(hbase, 'hbase')

    flume = cluster.get_service(flume_service_name)
    process_service(flume, 'flume')

    sqoop = cluster.get_service(sqoop_service_name)
    process_service(sqoop, 'sqoop')

    solr = cluster.get_service(solr_service_name)
    process_service(solr, 'solr')

    ks_indexer = cluster.get_service(ks_indexer_service_name)
    process_service(ks_indexer, 'ks_indexer')

    impala = cluster.get_service(impala_service_name)
    process_service(impala, 'impala')

    sentry = cluster.get_service(sentry_service_name)
    process_service(sentry, 'sentry')

    kms = cluster.get_service(kms_service_name)
    process_service(kms, 'kms')


if __name__ == '__main__':
    main()



# Copyright (c) 2014 Intel Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# The contents of this file are mainly copied from cm_api sources,
# released by Cloudera. Codes not used by Sahara CDH plugin are removed.
# You can find the original codes at
#
#     https://github.com/cloudera/cm_api/tree/master/python/src/cm_api
#
# To satisfy the pep8 and python3 tests, we did some changes to the codes.
# We also change some importings to use Sahara inherited classes.

from sahara_plugin_cdh.plugins.cdh.client import clusters
from sahara_plugin_cdh.plugins.cdh.client import cms
from sahara_plugin_cdh.plugins.cdh.client import hosts
from sahara_plugin_cdh.plugins.cdh.client import http_client
from sahara_plugin_cdh.plugins.cdh.client import resource
from sahara_plugin_cdh.plugins.cdh.client import users

API_AUTH_REALM = "Cloudera Manager"
API_CURRENT_VERSION = 8


class ApiResource(resource.Resource):
    """Top-level API Resource

    Resource object that provides methods for managing the top-level API
    resources.
    """

    def __init__(self, server_host, server_port=None,
                 username="admin", password="admin",
                 use_tls=False, version=API_CURRENT_VERSION):
        """Creates a Resource object that provides API endpoints.

        :param server_host: The hostname of the Cloudera Manager server.
        :param server_port: The port of the server. Defaults to 7180 (http) or
            7183 (https).
        :param username: Login name.
        :param password: Login password.
        :param use_tls: Whether to use tls (https).
        :param version: API version.
        :return: Resource object referring to the root.
        """
        self._version = version
        protocol = "https" if use_tls else "http"
        if server_port is None:
            server_port = 7183 if use_tls else 7180
        base_url = ("%s://%s:%s/api/v%s"
                    % (protocol, server_host, server_port, version))

        client = http_client.HttpClient(base_url)
        client.set_basic_auth(username, password, API_AUTH_REALM)
        client.set_headers({"Content-Type": "application/json"})
        resource.Resource.__init__(self, client)

    @property
    def version(self):
        """Returns the API version (integer) being used."""
        return self._version

    def get_cloudera_manager(self):
        """Returns a Cloudera Manager object."""
        return cms.ClouderaManager(self)

    def create_cluster(self, name, version=None, fullVersion=None):
        """Create a new cluster

        :param name: Cluster name.
        :param version: Cluster major CDH version, e.g. 'CDH5'. Ignored if
            fullVersion is specified.
        :param fullVersion: Complete CDH version, e.g. '5.1.2'. Overrides major
            version if both specified.
        :return: The created cluster.
        """
        return clusters.create_cluster(self, name, version, fullVersion)

    def get_all_clusters(self, view=None):
        """Retrieve a list of all clusters

        :param view: View to materialize ('full' or 'summary').
        :return: A list of ApiCluster objects.
        """
        return clusters.get_all_clusters(self, view)

    def get_cluster(self, name):
        """Look up a cluster by name

        :param name: Cluster name.
        :return: An ApiCluster object.
        """
        return clusters.get_cluster(self, name)

    def delete_host(self, host_id):
        """Delete a host by id

        :param host_id: Host id
        :return: The deleted ApiHost object
        """
        return hosts.delete_host(self, host_id)

    def get_all_hosts(self, view=None):
        """Get all hosts

        :param view: View to materialize ('full' or 'summary').
        :return: A list of ApiHost objects.
        """
        return hosts.get_all_hosts(self, view)

    def get_user(self, username):
        """Look up a user by username.

        @param username: Username to look up
        @return: An ApiUser object
        """
        return users.get_user(self, username)

    def update_user(self, user):
        """Update a user detail profile.

        @param user: An ApiUser object
        @return: An ApiUser object
        """
        return users.update_user(self, user)

    def get_service_health_status(self, cluster):
        """Get clusters service health status

        :param cluster: Cluster name.
        :return: A dict with cluster health status
        """
        cluster = clusters.get_cluster(self, cluster)
        return cluster.get_service_health_status()



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara_plugin_cdh.plugins.cdh import cloudera_utils as cu
from sahara_plugin_cdh.plugins.cdh.v5_13_0 import config_helper
from sahara_plugin_cdh.plugins.cdh.v5_13_0 import plugin_utils as pu
from sahara_plugin_cdh.plugins.cdh.v5_13_0 import validation


class ClouderaUtilsV5130(cu.ClouderaUtils):

    def __init__(self):
        cu.ClouderaUtils.__init__(self)
        self.pu = pu.PluginUtilsV5130()
        self.validator = validation.ValidatorV5130
        self.c_helper = config_helper.ConfigHelperV5130()

    def _load_version_specific_instance_configs(self, instance, default_conf):
        if instance.node_group.node_configs.get('HIVEMETASTORE', {}).get(
                'hive_enable_db_notification', None) is None:
            default_conf['HIVEMETASTORE']['hive_enable_db_notification'] = (
                'true')
